package no.ubicollab.osgi;

import java.math.BigInteger;
import java.rmi.RemoteException;
import java.security.*;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Date;
import java.net.*;
import javax.xml.rpc.ServiceException;

import no.ubicollab.axis.services.discoveryservice.*;
import no.ubicollab.axis.services.datastorageservice.*;

import no.ubicollab.osgi.service.collaboration.CollaborationService;

public class CollaborationImpl implements CollaborationService {
	
	private Connection sqlConn;
	private no.ubicollab.axis.services.discoveryservice.DiscoveryImpl discoveryProxy = null;
	private no.ubicollab.axis.services.datastorageservice.DataStorageImpl dataStorageProxy = null;

	public CollaborationImpl() {
		try {
			Class.forName("com.mysql.jdbc.Driver").newInstance();
			sqlConn = connectToDatabase();
			System.out.println("Connected to DB");
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		// TODO *** MUST BE ABLE TO GO THROUGH A SET OF DISCOVERY/DATA STORAGE SERVICES ***
		String discoveryServiceUrl = "http://localhost:8080/axis/services/discoveryService";
		String dataStorageServiceUrl = "http://localhost:8080/axis/services/dataStorageService";
		DiscoveryImplService discoveryService = new DiscoveryImplServiceLocator();
		DataStorageImplService dataStorageService = new DataStorageImplServiceLocator();
		
		try {
			discoveryProxy = discoveryService.getdiscoveryService(new URL(discoveryServiceUrl));
			dataStorageProxy = dataStorageService.getdataStorageService(new URL(dataStorageServiceUrl));
		} catch (ServiceException se) {
			se.printStackTrace();
		} catch (MalformedURLException mue) {
			mue.printStackTrace();
		}
	}

	/*
	 * Creates a collaboration instance with the specified collabInstName, and where the creator
	 * is specified by username. Returns the collaboration instance id. (non-Javadoc)
	 * 
	 * @see no.ubicollab.osgi.service.collaboration.CollaborationService#createCollaborationInstance(java.lang.String, java.lang.String)
	 */
	public String createCollaborationInstance(String username, String collabInstName) {
		Date date = new Date();
		String collabInstId = "";
		int count = 0;
		boolean personExists = checkIfPersonExists(username);
		
		if (personExists) {
			try {
				MessageDigest md5 = MessageDigest.getInstance("MD5");
				md5.update((username + date.toString()).getBytes());
				BigInteger bi = new BigInteger(md5.digest());
				collabInstId = bi.toString(16);
				if (collabInstId.startsWith("-")) {
					collabInstId = collabInstId.substring(1);
				}
			    if (collabInstId.length() % 2 != 0) {
			    	collabInstId = "0" + collabInstId;
			    }
				//System.out.println(collabInstId);
			    
				if (sqlConn.isClosed()) {
					sqlConn = connectToDatabase();
				}
					
				Statement s = sqlConn.createStatement();
				count = s.executeUpdate(
						   	"INSERT INTO collabInsts(collabInstId, name, creator)"
			               	+ " VALUES"
			               	+ "('" + collabInstId + "',"
			               	+ "'" + collabInstName + "',"
			               	+ "'" + username + "')");
				s.close ();
				
			} catch (NoSuchAlgorithmException nsae) {
				nsae.printStackTrace();
			} catch (SQLException sqle) {
				sqle.printStackTrace();
			}
		}
		
		if (count == 1) {
			addPersonToCollabInst(collabInstId, username);
			return collabInstId;
		} else {
			return "Error: CollabInst was not created";
		}
	}

	/*
	 * Removes a specified collaboration instance, if the username specified matches the
	 * collaboration instance's owner. Returns true if the collaboration instance was
	 * successfully removed, false otherwise. (non-Javadoc)
	 * 
	 * @see no.ubicollab.osgi.service.collaboration.CollaborationService#removeCollaborationInstance(java.lang.String, java.lang.String)
	 */
	public boolean removeCollaborationInstance(String collabInstId, String username) {
		int count = 0;
		String creator = "";
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			Statement s = sqlConn.createStatement();
			String query = "SELECT creator FROM collabInsts WHERE collabInstId = '" + collabInstId + "'";
			s.executeQuery(query);
		    ResultSet rs = s.getResultSet();
		    if (rs.next()) {
		    	creator = rs.getString("creator");
		    }
		    rs.close();
			s.close();
		    if (username.equals(creator)) {
		    	s = sqlConn.createStatement();
				count = s.executeUpdate("DELETE FROM collabInsts WHERE collabInstId = '" + collabInstId + "'");
				s.executeUpdate("DELETE FROM personCollabInst WHERE collabInstId = '" + collabInstId + "'");
				s.executeUpdate("DELETE FROM serviceCollabInst WHERE collabInstId = '" + collabInstId + "'");
				s.close ();
		    }		    
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}
		
		if (count == 1) {
	    	return true;
	    } else {		
	    	return false;
	    }
	}

	/*
	 * Adds a specified person to the specified collaboration instance. 
	 * Returns true if the person was successfully added to the collaboration instance,
	 * false otherwise. (non-Javadoc)
	 * 
	 * @see no.ubicollab.osgi.service.collaboration.CollaborationService#addPersonToCollabInst(java.lang.String, java.lang.String)
	 */
	public boolean addPersonToCollabInst(String collabInstId, String username) {
		// Sjekk at username eksisterer, ikke allerede er lagt til, og kobl sammen
		boolean personExists = checkIfPersonExists(username);
		boolean personInCollabInst = false;
		int count = 0;
		if (personExists) {
			personInCollabInst = checkIfPersonIsInCollabInst(collabInstId, username);
			if (!personInCollabInst) {
				try {
					if (sqlConn.isClosed()) {
						sqlConn = connectToDatabase();
					}
			    	Statement s = sqlConn.createStatement();
					count = s.executeUpdate(
								"INSERT INTO personCollabInst(username, collabInstId)"
								+ " VALUES"
								+ "('" + username + "',"
				               	+ "'" + collabInstId + "')");
					s.close ();
				} catch (SQLException sqle) {
					sqle.printStackTrace();
				} 
		    } 
		} 
		
		if (count == 1) {
	    	return true;
	    } else {		
	    	return false;
	    }
	}

	/*
	 * Removes a specified person from the specified collaboration instance.
	 * Returns true if the person was successfully removed from the collaboration instance,
	 * false otherwise. (non-Javadoc)
	 * 
	 * @see no.ubicollab.osgi.service.collaboration.CollaborationService#removePersonFromCollabInst(java.lang.String, java.lang.String)
	 */
	public boolean removePersonFromCollabInst(String collabInstId, String username) {
		// Sjekk at username er lagt til og fjern
		boolean personExists = checkIfPersonExists(username);
		boolean personInCollabInst = false;
		int count = 0;
		if (personExists) {
			personInCollabInst = checkIfPersonIsInCollabInst(collabInstId, username);
			if (personInCollabInst) {
				try {
					if (sqlConn.isClosed()) {
						sqlConn = connectToDatabase();
					}
			    	Statement s = sqlConn.createStatement();
					count = s.executeUpdate("DELETE FROM personCollabInst " +
								"WHERE collabInstId = '" + collabInstId + "' " +
								"AND username = '" + username + "'");
					s.close ();
				} catch (SQLException sqle) {
					sqle.printStackTrace();
				} 
		    } 
		} 
		
		if (count == 1) {
	    	return true;
	    } else {		
	    	return false;
	    }
	}

	/*
	 * Adds a specified service to the specified collaboration instance. 
	 * Returns true if the service was successfully added to the collaboration instance,
	 * false otherwise. (non-Javadoc)
	 * 
	 * @see no.ubicollab.osgi.service.collaboration.CollaborationService#addServiceToCollabInst(java.lang.String, java.lang.String)
	 */
	public boolean addServiceToCollabInst(String collabInstId, String descriptionUrl) {
		boolean serviceExists = checkIfServiceExists(descriptionUrl);
		boolean serviceInCollabInst = false;
		int count = 0;
		if (serviceExists) {
			serviceInCollabInst = checkIfServiceIsInCollabInst(collabInstId, descriptionUrl);
			if (!serviceInCollabInst) {
				try {
					if (sqlConn.isClosed()) {
						sqlConn = connectToDatabase();
					}
			    	Statement s = sqlConn.createStatement();
					count = s.executeUpdate(
								"INSERT INTO serviceCollabInst(descriptionUrl, collabInstId)"
								+ " VALUES"
								+ "('" + descriptionUrl + "',"
				               	+ "'" + collabInstId + "')");
					s.close ();
				} catch (SQLException sqle) {
					sqle.printStackTrace();
				} 
		    } 
		} 
		
		if (count == 1) {
	    	return true;
	    } else {		
	    	return false;
	    }
	}

	/*
	 * Removes a specified service from the specified collaboration instance. 
	 * Returns true if the service was successfully removed from the collaboration instance,
	 * false otherwise. (non-Javadoc)
	 * 
	 * @see no.ubicollab.osgi.service.collaboration.CollaborationService#removeServiceFromCollabInst(java.lang.String, java.lang.String)
	 */
	public boolean removeServiceFromCollabInst(String collabInstId, String descriptionUrl) {
		// Sjekk at ressurs er lagt til og fjern
		boolean serviceExists = checkIfServiceExists(descriptionUrl);
		boolean serviceInCollabInst = false;
		int count = 0;
		if (serviceExists) {
			serviceInCollabInst = checkIfServiceIsInCollabInst(collabInstId, descriptionUrl);
			if (serviceInCollabInst) {
				try {
					if (sqlConn.isClosed()) {
						sqlConn = connectToDatabase();
					}
			    	Statement s = sqlConn.createStatement();
					count = s.executeUpdate("DELETE FROM serviceCollabInst " +
								"WHERE collabInstId = '" + collabInstId + "' " +
								"AND descriptionUrl = '" + descriptionUrl + "'");
					s.close ();
				} catch (SQLException sqle) {
					sqle.printStackTrace();
				} 
		    } 
		} 
		
		if (count == 1) {
	    	return true;
	    } else {		
	    	return false;
	    }
	}

	/*
	 * Adds a specified file to the specified collaboration instance. 
	 * Returns true if the file was successfully added to the collaboration instance,
	 * false otherwise. (non-Javadoc)
	 * 
	 * @see no.ubicollab.osgi.service.collaboration.CollaborationService#addServiceToCollabInst(java.lang.String, java.lang.String)
	 */
	public boolean addFileToCollabInst(String collabInstId, String username, String appName, String fileName) {
		String fileId = username + "/" + appName + "/" + fileName;
		boolean fileExists = checkIfFileExists(username, appName, fileName);
		boolean fileInCollabInst = false;
		int count = 0;
		
		if (fileExists) {
			fileInCollabInst = checkIfFileIsInCollabInst(collabInstId, fileId);
			if (!fileInCollabInst) {
				try {
					if (sqlConn.isClosed()) {
						sqlConn = connectToDatabase();
					}
			    	Statement s = sqlConn.createStatement();
					count = s.executeUpdate(
								"INSERT INTO fileCollabInst(fileId, collabInstId)"
								+ " VALUES"
								+ "('" + fileId + "',"
				               	+ "'" + collabInstId + "')");
					s.close ();
				} catch (SQLException sqle) {
					sqle.printStackTrace();
				} 
		    } 
		} 
		
		if (count == 1) {
	    	return true;
	    } else {		
	    	return false;
	    }
	}

	/*
	 * Removes a specified file from the specified collaboration instance. 
	 * Returns true if the file was successfully removed from the collaboration instance,
	 * false otherwise. (non-Javadoc)
	 * 
	 * @see no.ubicollab.osgi.service.collaboration.CollaborationService#removeServiceFromCollabInst(java.lang.String, java.lang.String)
	 */
	public boolean removeFileFromCollabInst(String collabInstId, String username, String appName, String fileName) {
		String fileId = username + "/" + appName + "/" + fileName;
		boolean fileExists = checkIfFileExists(username, appName, fileName);
		boolean fileInCollabInst = false;
		int count = 0;
		
		if (fileExists) {
			fileInCollabInst = checkIfFileIsInCollabInst(collabInstId, fileId);
			if (fileInCollabInst) {
				try {
					if (sqlConn.isClosed()) {
						sqlConn = connectToDatabase();
					}
			    	Statement s = sqlConn.createStatement();
					count = s.executeUpdate(
								"DELETE FROM fileCollabInst "
								+ "WHERE fileId = '" + fileId + "' "
				               	+ "AND collabInstId = '" + collabInstId + "'");
					s.close ();
				} catch (SQLException sqle) {
					sqle.printStackTrace();
				} 
		    } 
		} 
		
		if (count == 1) {
	    	return true;
	    } else {		
	    	return false;
	    }
	}
	
	/*
	 * Gets all registered collaboration instances. 
	 * Returns an array of collaboration instance id's. (non-Javadoc)
	 * 
	 * @see no.ubicollab.osgi.service.collaboration.CollaborationService#getAllCollaborationInstances()
	 */
	public String[] getAllCollaborationInstances() {
		ArrayList<String> collabInstList = new ArrayList<String>();
		String[] collabInsts = null;
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			Statement s = sqlConn.createStatement();
			String query = "SELECT collabInstId FROM collabInsts";
			s.executeQuery(query);
		    ResultSet rs = s.getResultSet();
		    while (rs.next()) {
		    	String collabInstId = rs.getString("collabInstId");
		    	collabInstList.add(collabInstId);
		    }
		    rs.close();
			s.close();
			int noOfCollabInsts = collabInstList.size();
			collabInsts = new String[noOfCollabInsts];
			for (int i = 0; i < noOfCollabInsts; i++) {
				collabInsts[i] = collabInstList.get(i);
				//System.out.println("collabInsts[" + i + "]: " + collabInsts[i]);
			}
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}
		
		return collabInsts;
	}

	/*
	 * Gets all registered persons. 
	 * Returns an array of usernames. (non-Javadoc)
	 * 
	 * @see no.ubicollab.osgi.service.collaboration.CollaborationService#getAllPersons()
	 */
	public String[] getAllPersons() {
		ArrayList<String> personList = new ArrayList<String>();
		String[] persons = null;
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			Statement s = sqlConn.createStatement();
			String query = "SELECT username FROM persons";
			s.executeQuery(query);
		    ResultSet rs = s.getResultSet();
		    while (rs.next()) {
		    	String username = rs.getString("username");
		    	personList.add(username);
		    }
		    rs.close();
			s.close();
			int noOfPersons = personList.size();
			persons = new String[noOfPersons];
			for (int i = 0; i < noOfPersons; i++) {
				persons[i] = personList.get(i);
				//System.out.println("persons[" + i + "]: " + persons[i]);
			}
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}
		
		return persons;
	}

	/*
	 * Gets all registered services. 
	 * Returns an array of descriptionUrl's (non-Javadoc)
	 * 
	 * @see no.ubicollab.osgi.service.collaboration.CollaborationService#getAllServices()
	 */
	public String[] getAllServices() {
		String[] services = null;
		
		try {
			String serviceString = discoveryProxy.getService("");
			String[] servicesAndProtocols = serviceString.split(" ");
			int noOfServices = servicesAndProtocols.length / 2;
			services = new String[noOfServices];
			int servicesAndProtocolsIndex = 0;
			for (int i = 0; i < noOfServices; i++) {
				services[i] = servicesAndProtocols[servicesAndProtocolsIndex];
				servicesAndProtocolsIndex = servicesAndProtocolsIndex + 2;
			}
		} catch (RemoteException re) {
			re.printStackTrace();
		} catch (NullPointerException npe) {
			return null;
		}
		
		return services;
	}
	
	public String[] getPersonsFiles(String username) {
		String[] fileList = null;
		ArrayList<String> allFiles = new ArrayList<String>();
		try {
			String[] appList = dataStorageProxy.listAppNames(username);
			String appName;
			for (int i = 0; i < appList.length; i++) {
				appName = appList[i];
				String[] filesPerApp = dataStorageProxy.listFileNames(username, appName);
				String fileName;
				for (int j = 0; j < filesPerApp.length; j++) {
					fileName = filesPerApp[j];
					allFiles.add(username + "/" + appName + "/" + fileName);
				}
			}	
		} catch (RemoteException re) {
			re.printStackTrace();
		}
		
		int noOfFiles = allFiles.size();
		fileList = new String[noOfFiles];
		for (int i = 0; i < noOfFiles; i++) {
			fileList[i] = allFiles.get(i);
		}
		
		return fileList;
	}
	
	public String[] getPersonsCollaborationInstances(String username) {
		ArrayList<String> collabInstList = new ArrayList<String>();
		String[] collabInsts = null;
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			Statement s = sqlConn.createStatement();
			String query = "SELECT collabInstId FROM personCollabInst WHERE username = '" + username + "'";
			s.executeQuery(query);
		    ResultSet rs = s.getResultSet();
		    while (rs.next()) {
		    	String collabInstId = rs.getString("collabInstId");
		    	collabInstList.add(collabInstId);
		    }
		    rs.close();
			s.close();
			int noOfCollabInsts = collabInstList.size();
			collabInsts = new String[noOfCollabInsts];
			for (int i = 0; i < noOfCollabInsts; i++) {
				collabInsts[i] = collabInstList.get(i);
				//System.out.println("persons[" + i + "]: " + persons[i]);
			}
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}
		
		return collabInsts;
	}

	/*
	 * Gets all persons in the specified collaboration instance. 
	 * Returns an array of usernames associated with the collaboration instance. (non-Javadoc)
	 * 
	 * @see no.ubicollab.osgi.service.collaboration.CollaborationService#getAllPersonsInCollabInst(java.lang.String)
	 */
	public String[] getAllPersonsInCollabInst(String collabInstId) {
		ArrayList<String> personList = new ArrayList<String>();
		String[] persons = null;
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			Statement s = sqlConn.createStatement();
			String query = "SELECT username FROM personCollabInst WHERE collabInstId = '" + collabInstId + "'";
			s.executeQuery(query);
		    ResultSet rs = s.getResultSet();
		    while (rs.next()) {
		    	String username = rs.getString("username");
		    	personList.add(username);
		    }
		    rs.close();
			s.close();
			int noOfPersons = personList.size();
			persons = new String[noOfPersons];
			for (int i = 0; i < noOfPersons; i++) {
				persons[i] = personList.get(i);
				//System.out.println("persons[" + i + "]: " + persons[i]);
			}
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}
		
		return persons;
	}

	/*
	 * Gets all services in the specified collaboration instance. 
	 * Returns an array of descriptionUrl's associated with the collaboration instance. (non-Javadoc)
	 * 
	 * @see no.ubicollab.osgi.service.collaboration.CollaborationService#getAllServicesInCollabInst(java.lang.String)
	 */
	public String[] getAllServicesInCollabInst(String collabInstId) {
		ArrayList<String> serviceList = new ArrayList<String>();
		String[] services = null;
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			Statement s = sqlConn.createStatement();
			String query = "SELECT descriptionUrl FROM serviceCollabInst WHERE collabInstId = '" + collabInstId + "'";
			s.executeQuery(query);
		    ResultSet rs = s.getResultSet();
		    while (rs.next()) {
		    	String descriptionUrl = rs.getString("descriptionUrl");
		    	serviceList.add(descriptionUrl);
		    }
		    rs.close();
			s.close();
			int noOfServices = serviceList.size();
			services = new String[noOfServices];
			for (int i = 0; i < noOfServices; i++) {
				services[i] = serviceList.get(i);
			}
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}
		
		return services;
	}
	
	/*
	 * Gets all files in the specified collaboration instance. 
	 * Returns an array of fileId's associated with the collaboration instance. (non-Javadoc)
	 * 
	 * @see no.ubicollab.osgi.service.collaboration.CollaborationService#getAllServicesInCollabInst(java.lang.String)
	 */
	public String[] getAllFilesInCollabInst(String collabInstId) {
		ArrayList<String> fileList = new ArrayList<String>();
		String[] files = null;
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			Statement s = sqlConn.createStatement();
			String query = "SELECT fileId FROM fileCollabInst WHERE collabInstId = '" + collabInstId + "'";
			s.executeQuery(query);
		    ResultSet rs = s.getResultSet();
		    while (rs.next()) {
		    	String fileId = rs.getString("fileId");
		    	fileList.add(fileId);
		    }
		    rs.close();
			s.close();
			int noOfFiles = fileList.size();
			files = new String[noOfFiles];
			for (int i = 0; i < noOfFiles; i++) {
				files[i] = fileList.get(i);
			}
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}
		
		return files;
	}
	
	/*
	 * Gets the name of the specified collaboration instance. 
	 * Returns a string with the name of the collaboration instance. (non-Javadoc)
	 * 
	 * @see no.ubicollab.osgi.service.collaboration.CollaborationService#getCollabInstName(java.lang.String)
	 */
	public String getCollabInstName(String collabInstId) {
		String collabInstName = "";
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			Statement s = sqlConn.createStatement();
			String query = "SELECT name FROM collabInsts WHERE collabInstId = '" + collabInstId + "'";
			s.executeQuery(query);
		    ResultSet rs = s.getResultSet();
		    if (rs.next()) {
		    	collabInstName = rs.getString("name");
		    } else {
		    	collabInstName = "Error: No such id";
		    }
		    rs.close();
			s.close();		    
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}
		
		return collabInstName;
	}

	/*
	 * Gets the full name of the specified person. 
	 * Returns a string with the full name of the person. (non-Javadoc)
	 * 
	 * @see no.ubicollab.osgi.service.collaboration.CollaborationService#getPersonsFullName(java.lang.String)
	 */
	public String getPersonsFullName(String username) {
		String fullName = "";
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			Statement s = sqlConn.createStatement();
			String query = "SELECT fullName FROM persons WHERE username = '" + username + "'";
			s.executeQuery(query);
		    ResultSet rs = s.getResultSet();
		    if (rs.next()) {
		    	fullName = rs.getString("fullName");
		    } else {
		    	fullName = "Error: No such username";
		    }
		    rs.close();
			s.close();		    
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}
		
		return fullName;
	}
	
	/*
	 * Gets all registered information about the specified service. 
	 * Returns a string array of length 10, containing all registered information about
	 * the service. (non-Javadoc)
	 * 
	 * @see no.ubicollab.osgi.service.collaboration.CollaborationService#getInfoAboutService(java.lang.String)
	 */
	public String[] getInfoAboutService(String url) {
		String[] serviceInfo = null;
		
		try {
			serviceInfo = discoveryProxy.getInfoAboutService(url);
		} catch (RemoteException re) {
			re.printStackTrace();
		}
		
		return serviceInfo;
	}
	

	/*
	 * Checks if the specified person exists. Returns true if he or she exists, false otherwise.
	 */
	private boolean checkIfPersonExists(String username) {
		boolean personExists = false;
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			Statement s = sqlConn.createStatement();
			String query = "SELECT * FROM persons WHERE username = '" + username + "'";
			s.executeQuery(query);
		    ResultSet rs = s.getResultSet();
		    if (rs.next()) {
		    	personExists = true;
		    }
		    rs.close();
			s.close();
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}
		
		return personExists;
	}
	
	/*
	 * Checks if the specified person is in a particular collaboration instance.
	 * Returns true if he or she is, false otherwise.
	 */
	private boolean checkIfPersonIsInCollabInst(String collabInstId, String username) {
		boolean personInCollabInst = false;
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			Statement s = sqlConn.createStatement();
			String query = "SELECT username FROM personCollabInst WHERE collabInstId = '" + collabInstId + "'";
			s.executeQuery(query);
		    ResultSet rs = s.getResultSet();
		    while (rs.next()) {
		    	String person = rs.getString("username");
		    	if (person.equals(username)) {
		    		personInCollabInst = true;
		    		break;
		    	}
		    }
		    rs.close();
			s.close();
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}
		
		return personInCollabInst;
	}
	
	/*
	 * Checks if the specified service exists. Returns true if it does, false otherwise.
	 */
	private boolean checkIfServiceExists(String descriptionUrl) {
		boolean serviceExists = false;
		try {
			String service = discoveryProxy.getService("descriptionUrl:" + descriptionUrl);
			if (!service.equals("No matching services")) {
				serviceExists = true;
			}
		} catch (RemoteException re) {
			re.printStackTrace();
		}
		
		return serviceExists;
	}
	
	/*
	 * Checks if the specified service is in a particular collaboration instance.
	 * Returns true if it is, false otherwise.
	 */
	private boolean checkIfServiceIsInCollabInst(String collabInstId, String descriptionUrl) {
		boolean serviceInCollabInst = false;
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			Statement s = sqlConn.createStatement();
			String query = "SELECT descriptionUrl FROM serviceCollabInst WHERE collabInstId = '" + collabInstId + "'";
			s.executeQuery(query);
		    ResultSet rs = s.getResultSet();
		    while (rs.next()) {
		    	String service = rs.getString("descriptionUrl");
		    	if (service.equals(descriptionUrl)) {
		    		serviceInCollabInst = true;
		    		break;
		    	}
		    }
		    rs.close();
			s.close();
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}
		
		return serviceInCollabInst;
	}
	
	private boolean checkIfFileExists(String username, String appName, String fileName) {
		boolean fileExists = false;
		try {
			String[] files = dataStorageProxy.listFileNames(username, appName);
			for (String str : files) {
				if (str.equals(fileName)) {
					fileExists = true;
					break;
				}
			}
		} catch (RemoteException re) {
			re.printStackTrace();
		}
		
		return fileExists;
	}
	
	/*
	 * Checks if the specified file is in a particular collaboration instance.
	 * Returns true if it is, false otherwise.
	 */
	private boolean checkIfFileIsInCollabInst(String collabInstId, String fileId) {
		boolean fileInCollabInst = false;
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			Statement s = sqlConn.createStatement();
			String query = "SELECT fileId FROM fileCollabInst WHERE collabInstId = '" + collabInstId + "'";
			s.executeQuery(query);
		    ResultSet rs = s.getResultSet();
		    while (rs.next()) {
		    	String file = rs.getString("fileId");
		    	if (file.equals(fileId)) {
		    		fileInCollabInst = true;
		    		break;
		    	}
		    }
		    rs.close();
			s.close();
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}
		
		return fileInCollabInst;
	}
	
	/*
	 * Connects to the database. Returns the connection handle.
	 */
	private Connection connectToDatabase() {
		try {
			sqlConn = DriverManager.getConnection("jdbc:mysql://localhost/ubicollab", "ubi", "");
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}
		return sqlConn;
	}

	
	
	
}