package no.ubicollab.osgi;

import java.net.InetAddress;
import java.util.*;
import org.osgi.framework.*;

import no.ubicollab.osgi.service.datastorage.*;
import no.ubicollab.osgi.service.pocketdiscovery.*;

public class Activator implements BundleActivator {
	
	private BundleContext context = null;
	private DataStorageImpl dataStorage = null;
	private PocketDiscoveryService discoveryService;
	private String descriptionUrl;
    
    public void start(BundleContext bc) throws BundleException {
    	try {
    		context = bc;
	        Properties properties = new Properties();
	        properties.put("Language", "English");
	        properties.put("SOAP.service.name", "dataStorageService");
	        dataStorage = new DataStorageImpl();
	        
	        ServiceReference reference = context.getServiceReference(PocketDiscoveryService.class.getName());
	        if(reference != null){
	        	//Local pocket discovery service exists.
	        	//Start DataStorageService registering it in this discovery service using a local osgi call
	        	discoveryService = (PocketDiscoveryService)context.getService(reference);
	        	//gps=new GsmPositioningImpl(discoveryService, this);
	        	String serviceUrl = "http://" + InetAddress.getLocalHost().getHostAddress() + ":8080/axis/services/dataStorageService";
				descriptionUrl = serviceUrl + "?wsdl";
	        	discoveryService.registerService("", "", "UbiCollab Data Storage", "no.ubicollab.osgi.service.datastorage", "ws", descriptionUrl, serviceUrl, "");
	        	context.ungetService(reference);
	        }
	        else{
	        	//No local pocket discovery service exists
	        	//Start GpsPositioningImpl registering it in an external discoveryservice using a webservice call.
	        	//The url to the external discoveryserivce is retrieved from the configfile
	        	//gps=new GsmPositioningImpl(this);
	        }

	        context.registerService(DataStorageService.class.getName(), dataStorage, properties);
    	} catch (Exception e) {
    		throw new BundleException("Failed to start Data Storage Service", e);
    	}
    }
    
    public void stop(BundleContext bc) throws BundleException {
    	try {
    		discoveryService.removeService("name:UbiCollab Data Storage");
    		context = null;
    	} catch (Exception e) {
    		throw new BundleException("Failed to stop Data Storage Service", e);
    	}
    }
    
}