package no.ubicollab.osgi;

public class DiscoveryUtils {

	/**
	 * Builds a database query based on the contents of the search string.
	 * 
	 * @param searchString
	 *            tagId:&lt;tagId&gt; uuId:&lt;uuId&gt; name:&lt;name&gt;
	 *            type:&lt;type&gt; protocol:&lt;protocol&gt;
	 *            descriptionUrl:&lt;descriptionUrl&gt;
	 *            serviceUrl:&lt;serviceUrl&gt; Any number of these flags may be
	 *            present, but they must come in the right order. Example:
	 *            "name:My Service type:no.ubicollab.service"
	 * @return A database query string matching the input search string.
	 */
	public static String buildGetServiceQuery(String searchString) {
		// If the search string is empty, return all services
		if (searchString.equals("")) {
			return "SELECT * FROM discovery";
		}

		String tagId = null;
		String uuId = null;
		String name = null;
		String type = null;
		String protocol = null;
		String descriptionUrl = null;
		String serviceUrl = null;

		int tagIdIndex = searchString.indexOf("tagId:");
		int uuIdIndex = searchString.indexOf("uuId:");
		int nameIndex = searchString.indexOf("name:");
		int typeIndex = searchString.indexOf("type:");
		int protocolIndex = searchString.indexOf("protocol:");
		int descriptionUrlIndex = searchString.indexOf("descriptionUrl:");
		int serviceUrlIndex = searchString.indexOf("serviceUrl:");

		System.out.println(searchString);

		if (serviceUrlIndex != -1) {
			serviceUrl = searchString.substring(serviceUrlIndex + 11);
			if (serviceUrlIndex != 0) {
				searchString = searchString.substring(0, serviceUrlIndex - 1);
			}
		}
		if (descriptionUrlIndex != -1) {
			descriptionUrl = searchString.substring(descriptionUrlIndex + 15);
			if (descriptionUrlIndex != 0) {
				searchString = searchString.substring(0,
						descriptionUrlIndex - 1);
			}
		}
		if (protocolIndex != -1) {
			protocol = searchString.substring(protocolIndex + 9);
			if (protocolIndex != 0) {
				searchString = searchString.substring(0, protocolIndex - 1);
			}
		}
		if (typeIndex != -1) {
			type = searchString.substring(typeIndex + 5);
			if (typeIndex != 0) {
				searchString = searchString.substring(0, typeIndex - 1);
			}
		}
		if (nameIndex != -1) {
			name = searchString.substring(nameIndex + 5);
			if (nameIndex != 0) {
				searchString = searchString.substring(0, nameIndex - 1);
			}
		}
		if (uuIdIndex != -1) {
			uuId = searchString.substring(uuIdIndex + 5);
			if (uuIdIndex != 0) {
				searchString = searchString.substring(0, uuIdIndex - 1);
			}
		}
		if (tagIdIndex != -1) {
			tagId = searchString.substring(tagIdIndex + 6);
			// searchString = searchString.substring(0, tagIdIndex);
		}

		String query = "SELECT * FROM discovery WHERE ";
		boolean firstFlag = true;
		if (tagId != null) {
			query += "tagId = '" + tagId + "' ";
			firstFlag = false;
		}
		if (uuId != null) {
			if (!firstFlag) {
				query += "AND ";
			}
			query += "uuId = '" + uuId + "' ";
			firstFlag = false;
		}
		if (name != null) {
			if (!firstFlag) {
				query += "AND ";
			}
			query += "name LIKE '%" + name + "%' ";
			firstFlag = false;
		}
		if (type != null) {
			if (!firstFlag) {
				query += "AND ";
			}
			if (type.startsWith("*") && type.endsWith("*")) {
				type = type.substring(1, type.length() - 1);
				query += "type LIKE '%" + type + "%' ";
				firstFlag = false;
			} else if (type.startsWith("*")) {
				type = type.substring(1);
				query += "type LIKE '%" + type + "' ";
				firstFlag = false;
			} else if (type.endsWith("*")) {
				type = type.substring(0, type.length() - 1);
				query += "type LIKE '" + type + "%' ";
				firstFlag = false;
			} else {
				query += "type = '" + type + "' ";
				firstFlag = false;
			}

		}
		if (protocol != null) {
			if (!firstFlag) {
				query += "AND ";
			}
			query += "protocol = '" + protocol + "' ";
			firstFlag = false;
		}
		if (descriptionUrl != null) {
			if (!firstFlag) {
				query += "AND ";
			}
			query += "descriptionUrl = '" + descriptionUrl + "' ";
			firstFlag = false;
		}
		if (serviceUrl != null) {
			if (!firstFlag) {
				query += "AND ";
			}
			query += "serviceUrl = '" + serviceUrl + "' ";
			firstFlag = false;
		}

		// If the search string was erroneous, return no services
		if (query.endsWith("WHERE ")) {
			query = "";
		}

		System.out.println(query);

		return query;
	}

	/**
	 * Builds a database query based on the content of the remove string.
	 * 
	 * @param removeString
	 *            tagId:&lt;tagId&gt; uuId:&lt;uuId&gt; name:&lt;name&gt;
	 *            type:&lt;type&gt; protocol:&lt;protocol&gt;
	 *            descriptionUrl:&lt;descriptionUrl&gt;
	 *            serviceUrl:&lt;serviceUrl&gt; Any number of these flags may be
	 *            present, but they must come in the right order. Example:
	 *            "name:My Service type:no.ubicollab.service"
	 * @return A database query string matching the input remove string.
	 */
	public static String buildRemoveServiceQuery(String removeString) {
		String tagId = null;
		String uuId = null;
		String name = null;
		String type = null;
		String protocol = null;
		String descriptionUrl = null;
		String serviceUrl = null;

		int tagIdIndex = removeString.indexOf("tagId:");
		int uuIdIndex = removeString.indexOf("uuId:");
		int nameIndex = removeString.indexOf("name:");
		int typeIndex = removeString.indexOf("type:");
		int protocolIndex = removeString.indexOf("protocol:");
		int descriptionUrlIndex = removeString.indexOf("descriptionUrl:");
		int serviceUrlIndex = removeString.indexOf("serviceUrl:");

		System.out.println(removeString);

		if (serviceUrlIndex != -1) {
			serviceUrl = removeString.substring(serviceUrlIndex + 11);
			if (serviceUrlIndex != 0) {
				removeString = removeString.substring(0, serviceUrlIndex - 1);
			}
		}
		if (descriptionUrlIndex != -1) {
			descriptionUrl = removeString.substring(descriptionUrlIndex + 15);
			if (descriptionUrlIndex != 0) {
				removeString = removeString.substring(0,
						descriptionUrlIndex - 1);
			}
		}
		if (protocolIndex != -1) {
			protocol = removeString.substring(protocolIndex + 9);
			if (protocolIndex != 0) {
				removeString = removeString.substring(0, protocolIndex - 1);
			}
		}
		if (typeIndex != -1) {
			type = removeString.substring(typeIndex + 5);
			if (typeIndex != 0) {
				removeString = removeString.substring(0, typeIndex - 1);
			}
		}
		if (nameIndex != -1) {
			name = removeString.substring(nameIndex + 5);
			if (nameIndex != 0) {
				removeString = removeString.substring(0, nameIndex - 1);
			}
		}
		if (uuIdIndex != -1) {
			uuId = removeString.substring(uuIdIndex + 5);
			if (uuIdIndex != 0) {
				removeString = removeString.substring(0, uuIdIndex - 1);
			}
		}
		if (tagIdIndex != -1) {
			tagId = removeString.substring(tagIdIndex + 6);
			// removeString = removeString.substring(0, tagIdIndex);
		}

		String query = "DELETE FROM discovery WHERE ";
		boolean firstFlag = true;
		if (tagId != null) {
			query += "tagId = '" + tagId + "' ";
			firstFlag = false;
		}
		if (uuId != null) {
			if (!firstFlag) {
				query += "AND ";
			}
			query += "uuId = '" + uuId + "' ";
			firstFlag = false;
		}
		if (name != null) {
			if (!firstFlag) {
				query += "AND ";
			}
			query += "name = '" + name + "' ";
			firstFlag = false;
		}
		if (type != null) {
			if (!firstFlag) {
				query += "AND ";
			}
			query += "type = '" + type + "' ";
			firstFlag = false;
		}
		if (protocol != null) {
			if (!firstFlag) {
				query += "AND ";
			}
			query += "protocol = '" + protocol + "' ";
			firstFlag = false;
		}
		if (descriptionUrl != null) {
			if (!firstFlag) {
				query += "AND ";
			}
			query += "descriptionUrl = '" + descriptionUrl + "' ";
			firstFlag = false;
		}
		if (serviceUrl != null) {
			if (!firstFlag) {
				query += "AND ";
			}
			query += "serviceUrl = '" + serviceUrl + "' ";
			firstFlag = false;
		}

		System.out.println(query);

		return query;
	}

}
