package no.ubicollab.osgi;

import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.*;

public class ServiceRemovalThread extends Thread {
	
	private static boolean running = true;
	private DiscoveryImpl discovery = null;
	private Connection sqlConn = null;
	private String query = "SELECT * FROM discovery";
	private ArrayList<String> serviceList = null;
	
	public ServiceRemovalThread(DiscoveryImpl discovery, Connection sqlConn) {
		this.discovery = discovery;
		this.sqlConn = sqlConn;
		serviceList = new ArrayList<String>();
	}
	
	public void run() {
		while (running) {
			serviceList.clear();
			try {
				if (sqlConn.isClosed()) {
					sqlConn = discovery.connectToDatabase();
				}
				Statement s = sqlConn.createStatement();
				s.executeQuery(query);
			    ResultSet rs = s.getResultSet();
			    while (rs.next()) {
			    	String service = rs.getString("descriptionUrl");
			    	serviceList.add(service);
			    }
			    rs.close();
				s.close();
			} catch (SQLException sqle) {
				sqle.printStackTrace();
			}
			
			for (String service : serviceList) {
				boolean alive = checkIfServiceIsAlive(service);
		    	if (!alive) {
		    		removeService(service);
		    	}
			}
			
			try {
				Thread.sleep(18000000);
			} catch (InterruptedException ie) {
				ie.printStackTrace();
			}
		}
	}
	
	private boolean checkIfServiceIsAlive(String descriptionUrl) {
		boolean alive = true;
		try {
			URL url = new URL(descriptionUrl);
			URLConnection conn = url.openConnection();
			int responseCode = ((HttpURLConnection)conn).getResponseCode();
			if (responseCode != 200) {
				alive = false;
			}
		} catch (Exception e) {
			alive = false;
		}
		System.out.println(new Date() +": " + descriptionUrl + " is alive: " + alive);
		return alive;
	}
	
	private void removeService(String descriptionUrl) {
		System.out.println("Removing " + descriptionUrl);
		try {
			if (sqlConn.isClosed()) {
				sqlConn = discovery.connectToDatabase();
			}
			Statement s = sqlConn.createStatement();
			String query = "DELETE FROM discovery WHERE descriptionUrl = '" + descriptionUrl + "'";
			s.executeUpdate(query);
			s.close();
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}
	}
	
	public static void stopThread() {
		running = false;
		System.out.println("Service removal process is stopped");
	}

}
