package no.ubicollab.osgi;

import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.rmi.RemoteException;

import javax.xml.rpc.ServiceException;

import no.ubicollab.osgi.service.pocketdiscovery.PocketDiscoveryService;
import no.ubicollab.osgi.service.positioning.gps.PositioningPluginService;
import no.ubicollab.axis.services.discoveryService.*;



public class GpsPositioningImpl implements PositioningPluginService {
	
	private Activator activator;
	private DiscoveryImpl discoveryServiceProxy;
	private PocketDiscoveryService discoveryService;
	private ConfigFileManager configFileManager;
	SerialPortGPS gpsConnection;
	
	//Will be overwritten with data from configfile 
	String ownerUsername= "andreabr";
	String gpsComPort= "COM7:";
	String discoveryServiceUrl = "http://";
	
	//Start GpsPositioningImpl registering it in an external discoveryservice using a webservice call.
	//The url to the external discoveryserivce is retrieved from the configfile
	public GpsPositioningImpl(Activator activator){
		this.activator=activator;
		
		configFileManager = new ConfigFileManager();
		//configFileManager.writeConfigFile("andreabr","COM7","http://something");
		String[] config = configFileManager.readConfigFile();
		if(config==null){
			System.out.println("GpsPositioningImpl: There was an error reading the config file. Stopping...");
			activator.stop(Activator.bc);
		}
		else{
			ownerUsername= config[0];
			gpsComPort= config[1];
			discoveryServiceUrl = config[2];
			
			//create the proxy to the discoveryService webservice
			DiscoveryImplServiceLocator service = new DiscoveryImplServiceLocator();
			try {
				if(discoveryServiceUrl!=null)
					discoveryServiceProxy = service.getdiscoveryService(new java.net.URL(discoveryServiceUrl));
				else{
					System.out.println("Could not find textfile. Using http://gakkan.no:8080/discoveryService as default discoveryService location");
					discoveryServiceProxy = service.getdiscoveryService(new java.net.URL("http://gakkan.no:8080/discoveryService"));
				}
				
				//Register this service in the discoveryService
				String serviceUrl="http://"+InetAddress.getLocalHost().getHostAddress()+":8080/axis/services/gpsPositioningService";
				String descriptionUrl=serviceUrl+"?wsdl";
				discoveryServiceProxy.registerService("","","GpsPositioningService","no.ubicollab.osgi.service.positioning.gps","ws",descriptionUrl,serviceUrl,ownerUsername);
				
				//Start GPS bluetooth (virtual serial port) connection 
				//config[1] is the comport of the gps
				gpsConnection = new SerialPortGPS(gpsComPort);
				
			} catch (MalformedURLException e) {
				e.printStackTrace();
			} catch (ServiceException e) {
				e.printStackTrace();
			} catch (UnknownHostException e) {
				e.printStackTrace();
			} catch (RemoteException e) {
				e.printStackTrace();
			}
		}
		
	}
	
	//Start GpsPositioningImpl registering it in the local discovery service using a osgi call
	public GpsPositioningImpl(PocketDiscoveryService discoveryService, Activator activator){
		this.activator = activator;
		this.discoveryService = discoveryService;
		configFileManager = new ConfigFileManager();
		//configFileManager.writeConfigFile("andreabr","COM7","http://something");
		String[] config = configFileManager.readConfigFile();
		if(config==null){
			System.out.println("GpsPositioningImpl: There was an error reading the config file. Stopping...");
			activator.stop(Activator.bc);
		}
		else{
			ownerUsername= config[0];
			gpsComPort= config[1];
			discoveryServiceUrl = config[2];
			
			try {
				//Register this service in the discoveryService
				String serviceUrl="http://"+InetAddress.getLocalHost().getHostAddress()+":8080/axis/services/gpsPositioningService";
				String descriptionUrl=serviceUrl+"?wsdl";
				discoveryService.registerService("","","GpsPositioningService","no.ubicollab.osgi.service.positioning.gps","ws",descriptionUrl,serviceUrl,ownerUsername);
				//Start GPS bluetooth (virtual serial port) connection 
				//config[1] is the comport of the gps
				gpsConnection = new SerialPortGPS(gpsComPort);
			} 
			catch (UnknownHostException e) {
				e.printStackTrace();
			}
		}
	}

	public double[] getPosition(String username) {
		System.out.println("<GpsPositioning>getting pos of "+username);
		//System.out.println("owner is "+ownerUsername);
		if(!ownerUsername.equals(username)){
			System.out.println("<GpsPositioning>Don't have the position of this user: "+username);
			return null;
		}
		LocationData pos = gpsConnection.getPosition();
		try{
			double longitude = pos.getLongAsDecimalDegrees();
			double latitude = pos.getLatAsDecimalDegrees();
			double alt=0;
			double timestamp=0;
			if(pos.altitude!=null)
				alt = Double.parseDouble(pos.altitude);
			if(pos.time!=null)
				timestamp=Double.parseDouble(pos.time);
			System.out.println("<GpsPositioning>Returning data...");
			System.out.println("<GpsPositioning>Longitude: "+longitude);
			System.out.println("<GpsPositioning>Latitude: "+latitude);
			System.out.println("<GpsPositioning>Altitude: "+alt);
			System.out.println("<GpsPositioning>Timestamp: "+timestamp);
			return new double[] {longitude, latitude, alt, timestamp};
		}
		catch(NumberFormatException nfe){
			System.out.println("<GpsPositioning>Could not convert string to double");
			nfe.printStackTrace();
			return null;
		}
		
	}
	
	public void stop(){
		try{
			//Try local osgi call to pocketdiscoveryService
			if(discoveryService!=null){
				discoveryService.removeService("descriptionUrl:"+"http://"+InetAddress.getLocalHost().getHostAddress()+":8080/axis/services/gpsPositioningService?wsdl");
			}
			//Try webservice call to discoveryService
			else{
				discoveryServiceProxy.removeService("descriptionUrl:"+"http://"+InetAddress.getLocalHost().getHostAddress()+":8080/axis/services/gpsPositioningService?wsdl");
			}
		}
		catch(Exception e){
			System.out.println("<GpsPositioning>Unable to unregister myself from discoveryservice");
		}
	}
	
}
