package no.ubicollab.osgi;

import java.util.NoSuchElementException;
import java.util.StringTokenizer;


/**
 * Parser for the $(GP)RMC command of the NMEA-0183 GPS standard.
 * See <a href="http://vancouver-webpages.com/peter/nmeafaq.txt">http://vancouver-webpages.com/peter/nmeafaq.txt</a> for details
 * @author Andreas Brustad
 *
 */
public class Parser
{
	public static final String WARNING="V";

	public Parser()
	{
	}
	/**
	 * Parses the $GPRMC command and returns the positiondata as a LocationData object. As the GPRMC command does not contain altitude, this is set to "0". 
	 * @param string The $GPRMC string to parse.
	 * @return a LocationData object with the position, or null if parsing failed or the GPS reported that the data was inaccurate (Warning field set to Parser.WARNING).
	 * @see LocationData
	 */
	public static LocationData parse(String string)
	{
		StringTokenizer st =new StringTokenizer(string, ",");

		try
		{
		
			String type=st.nextToken();
			//Discard if not $GPRMC
			if(!type.equals("$GPRMC"))
				return null;

			String time=st.nextToken();
			
			String warning=st.nextToken();
			//Discard if not valid position
			if(Parser.WARNING.equals(warning))
				return null;
		
			LocationData newLocData =new LocationData();
			newLocData.type=type;
			//newLocData.time=time.substring(0,4);
			newLocData.warning=warning;
			newLocData.latitude=st.nextToken();
			newLocData.latDir =st.nextToken();
			newLocData.longitude=st.nextToken();
			newLocData.longDir =st.nextToken();
			newLocData.time=time;
			//don't need the speed
			//don't need the course
			st.nextToken();
			st.nextToken();
			newLocData.date=st.nextToken();

			return newLocData;
		}
		catch(NoSuchElementException nse)
		{
			System.out.println("parse exception");
			return null;
		}
	}
}