package no.ubicollab.osgi;

import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.rmi.RemoteException;
import java.util.Hashtable;

import javax.xml.rpc.ServiceException;

import no.ubicollab.osgi.service.pocketdiscovery.PocketDiscoveryService;
import no.ubicollab.osgi.service.positioning.gsm.PositioningPluginService;
import no.ubicollab.axis.services.discoveryService.*;

/**
* This class has been implemented to test the plugin functionality of the Positioning Service. 
* Therefore only the plugin functionality is implemented. The getPosition-method does only return hardcoded positions for some users. 
* 
* 
* @author Andreas Brustad
* @see no.ubicollab.osgi.service.positioning.PositioningService
*
*/
public class GsmPositioningImpl implements PositioningPluginService {
	
	private Activator activator;
	private DiscoveryImpl discoveryServiceProxy;
	private PocketDiscoveryService discoveryService;
	private ConfigFileManager configFileManager;
	
	private Hashtable hardcodedPositions;
	
	//Will be overwritten with data from configfile 
	private String discoveryServiceUrl = "http://";
	
	//Start GpsPositioningImpl registering it in an external discoveryservice using a webservice call.
	//The url to the external discoveryserivce is retrieved from the configfile
	public GsmPositioningImpl(Activator activator){
		this.activator=activator;
		
		hardcodedPositions = new Hashtable();
		fillTableWithHardcodedPositions();
		
		configFileManager = new ConfigFileManager();
		//configFileManager.writeConfigFile("andreabr","COM7","http://something");
		String[] config = configFileManager.readConfigFile();
		if(config==null){
			System.out.println("<GsmPositioning>There was an error reading the config file. Stopping...");
			activator.stop(Activator.bc);
		}
		else{
			discoveryServiceUrl = config[0];
			
			//create the proxy to the discoveryService webservice
			DiscoveryImplServiceLocator service = new DiscoveryImplServiceLocator();
			try {
				if(discoveryServiceUrl!=null)
					discoveryServiceProxy = service.getdiscoveryService(new java.net.URL(discoveryServiceUrl));
				else{
					System.out.println("<GsmPositioning>Could not find textfile. Using http://gakkan.no:8080/discoveryService as default discoveryService location");
					discoveryServiceProxy = service.getdiscoveryService(new java.net.URL("http://gakkan.no:8080/discoveryService"));
				}
				
				//Register this service in the discoveryService
				String serviceUrl="http://"+InetAddress.getLocalHost().getHostAddress()+":8080/axis/services/gsmPositioningService";
				String descriptionUrl=serviceUrl+"?wsdl";
				discoveryServiceProxy.registerService("","","GsmPositioningService","no.ubicollab.osgi.service.positioning.gsm","ws",descriptionUrl,serviceUrl,"");
				
				
			} catch (MalformedURLException e) {
				e.printStackTrace();
			} catch (ServiceException e) {
				e.printStackTrace();
			} catch (UnknownHostException e) {
				e.printStackTrace();
			} catch (RemoteException e) {
				e.printStackTrace();
			}
		}
		
	}
	
	//Start GpsPositioningImpl registering it in the local discovery service using a osgi call
	public GsmPositioningImpl(PocketDiscoveryService discoveryService, Activator activator){
		this.activator = activator;
		this.discoveryService = discoveryService;
		
		hardcodedPositions = new Hashtable();
		fillTableWithHardcodedPositions();
		
		/*
		configFileManager = new ConfigFileManager();
		//configFileManager.writeConfigFile("andreabr","COM7","http://something");
		String[] config = configFileManager.readConfigFile();
		if(config==null){
			System.out.println("GsmPositioningImpl: There was an error reading the config file. Stopping...");
			activator.stop(Activator.bc);
		}
		else{
			discoveryServiceUrl = config[0];
		*/	
		try {
			//Register this service in the discoveryService
			String serviceUrl="http://"+InetAddress.getLocalHost().getHostAddress()+":8080/axis/services/gsmPositioningService";
			String descriptionUrl=serviceUrl+"?wsdl";
			discoveryService.registerService("","","GsmPositioningService","no.ubicollab.osgi.service.positioning.gsm","ws",descriptionUrl,serviceUrl,"");

		} 
		catch (UnknownHostException e) {
			System.out.println("<GsmPositioning>Error registating service in pocketDiscovery service");
			e.printStackTrace();
		}
		//}
	}

	/**
	 * This method is supposed to retrieve a users position from the GSM-network through a Telenor web-service or similar.
	 * This is currently not implemented. Only hardcoded positions are returned
	 * @param username The username of the user to be located
	 * @return a double[] of length 3. Where [0] = longitude, [1] = latitude, [2] = altitude, or null if the position of this user is not found.
	 * @see no.ubicollab.osgi.service.positioning.gsm.PositioningPluginService#getPosition(java.lang.String)
	 */
	public double[] getPosition(String username) {
		System.out.println("<GsmPositioning>getting pos of "+username);
		
		LocationData loc = (LocationData)hardcodedPositions.get(username);
		if(loc == null){
			System.out.println("<GsmPositioning>Don't have the position of this user: "+username);
			return null;
		}
		
		try{
			double longitude = loc.getLongAsDecimalDegrees();
			double latitude = loc.getLatAsDecimalDegrees();
			double alt=0;
			double timestamp= 0;
			if(loc.altitude!=null)
				alt = Double.parseDouble(loc.altitude);
			if(loc.time!=null)
				timestamp=Double.parseDouble(loc.time);
			System.out.println("<GsmPositioning>Returning data...");
			System.out.println("<GsmPositioning>Longitude: "+longitude);
			System.out.println("<GsmPositioning>Latitude: "+latitude);
			System.out.println("<GsmPositioning>Altitude: "+alt);
			System.out.println("<GsmPositioning>Timestamp: "+timestamp);
			return new double[] {longitude, latitude, alt, timestamp};
		}
		catch(NumberFormatException nfe){
			System.out.println("Could not altitude string to double");
			nfe.printStackTrace();
			return null;
		}
		
	}
	
	private void fillTableWithHardcodedPositions(){
		hardcodedPositions.put("andreabr",new LocationData("01022.9505", "E", "6327.0725", "N", "200.11"));
		hardcodedPositions.put("mosveen", new LocationData("01024.4723", "E", "6325.4723", "N", "3250.53"));
		hardcodedPositions.put("monica", new LocationData("01023.6979", "E", "6325.8308", "N", "15.00"));
		hardcodedPositions.put("babak", new LocationData("01026.2258", "E", "6325.4047", "N", "100.00"));
		hardcodedPositions.put("georgebush", new LocationData("07700.0300", "W", "3800.9000", "N", "50.39"));
	}
	
	public void stop(){
		try{
			//Try local osgi call to pocketdiscoveryService
			if(discoveryService!=null){
				discoveryService.removeService("descriptionUrl:"+"http://"+InetAddress.getLocalHost().getHostAddress()+":8080/axis/services/gsmPositioningService?wsdl");
			}
			//Try webservice call to discoveryService
			else{
				discoveryServiceProxy.removeService("descriptionUrl:"+"http://"+InetAddress.getLocalHost().getHostAddress()+":8080/axis/services/gsmPositioningService?wsdl");
			}
		}
		catch(Exception e){
			System.out.println("<GsmPositioning>Unable to unregister myself from discoveryservice");
		}
	}
	
}
