package no.ubicollab.osgi;


import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.xml.rpc.ServiceException;

import no.ubicollab.axis.services.discoveryService.DiscoveryImpl;
import no.ubicollab.axis.services.discoveryService.DiscoveryImplServiceLocator;
import no.ubicollab.osgi.service.location.LocationService;
import no.ubicollab.osgi.service.pocketdiscovery.PocketDiscoveryService;



public class LocationImpl implements LocationService {
	
	private FileAccess fileAccess;
	private Hashtable locations;
	
	private Activator activator;
	private DiscoveryImpl discoveryServiceProxy;
	private PocketDiscoveryService discoveryService;
	private ConfigFileManager configFileManager;
	
	//Will be overwritten with data from configfile 
	private String discoveryServiceUrl = "http://";
	
	
	//Start LocationImpl registering it in an external discoveryservice using a webservice call.
	//The url to the external discoveryserivce is retrieved from the configfile
	public LocationImpl(Activator activator){
		this.activator=activator;
		
		//Read location zones from file
		fileAccess = new FileAccess();
		locations= (Hashtable)fileAccess.readObjectAndClose();
		if(locations ==null){
			System.out.println("<LocationService>Nothing read from file. Creating new location list");
			locations=new Hashtable();
		}
		
		
		//register in discoveryService
		configFileManager = new ConfigFileManager();
		//configFileManager.writeConfigFile("andreabr","COM7","http://something");
		String[] config = configFileManager.readConfigFile();
		if(config==null){
			System.out.println("<LocationService>There was an error reading the config file. Stopping...");
			activator.stop(Activator.bc);
		}
		else{
			discoveryServiceUrl = config[0];
			
			//create the proxy to the discoveryService webservice
			DiscoveryImplServiceLocator service = new DiscoveryImplServiceLocator();
			try {
				if(discoveryServiceUrl!=null)
					discoveryServiceProxy = service.getdiscoveryService(new java.net.URL(discoveryServiceUrl));
				else{
					System.out.println("<LocationService>Could not find configfile. Using http://gakkan.no:8080/discoveryService as default discoveryService location");
					discoveryServiceProxy = service.getdiscoveryService(new java.net.URL("http://gakkan.no:8080/discoveryService"));
				}
				
				//Register this service in the discoveryService
				String serviceUrl="http://"+InetAddress.getLocalHost().getHostAddress()+":8080/axis/services/locationService";
				String descriptionUrl=serviceUrl+"?wsdl";
				discoveryServiceProxy.registerService("","","LocationService","no.ubicollab.osgi.service.location","ws",descriptionUrl,serviceUrl,"");
				
				
			} catch (MalformedURLException e) {
				e.printStackTrace();
			} catch (ServiceException e) {
				e.printStackTrace();
			} catch (UnknownHostException e) {
				e.printStackTrace();
			} catch (RemoteException e) {
				e.printStackTrace();
			}
		}
		
	}
	
	//Start LocationImpl registering it in the local discovery service using a osgi call
	public LocationImpl(PocketDiscoveryService discoveryService, Activator activator){
		this.activator=activator;
		this.discoveryService = discoveryService;
		//Read location zones from file
		fileAccess = new FileAccess();
		locations= (Hashtable)fileAccess.readObjectAndClose();
		if(locations ==null){
			System.out.println("<LocationService>Nothing read from file. Creating new location list");
			locations=new Hashtable();
		}
//		For testing only
		//addLocation("itbygget",new double[]{10.401936,10.403053,10.403793,10.403311,10.403560,10.403246}, new double[]{63.417278,63.416183,63.416441,63.416785,63.416850,63.417503},0);
		//addLocation("ITbygget",new double[]{10.401284,10.402164,10.403793,10.403311,10.403560,10.403246}, new double[]{63.417463,63.415922,63.416441,63.416785,63.416850,63.417503},0);
		//addLocation("Telenor Tyholt",new double[]{10.430746,10.425810,10.439028,10.444650}, new double[]{63.425485,63.421395,63.418764,63.424333},0);
		//addLocation("Midtbyen",new double[]{10.376672,10.393066,10.405855}, new double[]{63.431167,63.423718,63.436963},0);
		//addLocation("Parkveien 28",new double[]{10.406262,10.406348,10.410382,10.409867}, new double[]{63.425283,63.423997,63.423958,63.425350},0);
		//addLocation("Munkholmen",new double[]{10.378475,10.377960,10.387487,10.389032}, new double[]{63.454191,63.449895,63.449665,63.455879},0);
		
		//saveLocationListToFile();
		//System.out.println("<LocationService> Saved");
		
		
		
		System.out.println("Contains"+ getLocation(10.402971666,63.4169816665));
		
		for (Enumeration e = locations.elements() ; e.hasMoreElements() ;) {
	         Location loc= (Location)e.nextElement();
	         int[] x = loc.area.xpoints;
	         int[] y = loc.area.ypoints;
	         System.out.println("<Locationservice>"+loc.getName()+ "points"+loc.area.npoints);
	         for(int i=0; i<x.length; i++){
	        	 System.out.println("<LocationService>X: "+x[i]);
	        	 System.out.println("<LocationService>Y: "+y[i]);
	         }
		}
		
		try {
			//Register this service in the discoveryService
			String serviceUrl="http://"+InetAddress.getLocalHost().getHostAddress()+":8080/axis/services/locationService";
			String descriptionUrl=serviceUrl+"?wsdl";
			discoveryService.registerService("","","LocationService","no.ubicollab.osgi.service.location","ws",descriptionUrl,serviceUrl,"");

		} 
		catch (UnknownHostException e) {
			System.out.println("<LocationService>Error registating service in pocketDiscovery service");
			e.printStackTrace();
		}
	}
	
	

	public boolean addLocation(String locationName, double[] longitudes, double[] latitudes, int altitude) {
		locations.put(locationName, new Location(locationName, longitudes, latitudes, altitude));
		saveLocationListToFile();
		return true;
	}

	public boolean removeLocatoin(String locationName) {
		locations.remove(locationName);
		saveLocationListToFile();
		return true;
	}

	public String getLocation(double longitude, double latitude, int altitude, int altitudeErrorMargin) {
		for (Enumeration e = locations.elements() ; e.hasMoreElements() ;) {
	         Location loc= (Location)e.nextElement();
	         if(loc.containsPoint(longitude, latitude, altitude, altitudeErrorMargin))
	        	 return loc.getName();
	     }
		return null;
	}

	public String getLocation(double longitude, double latitude) {
		for (Enumeration e = locations.elements() ; e.hasMoreElements() ;) {
	         Location loc= (Location)e.nextElement();
	         if(loc.containsPoint(longitude, latitude))
	        	 return loc.getName();
	     }
		return null;
	}
	public void saveLocationListToFile(){
		if(fileAccess.writeObjectAndClose(locations))
			System.out.println("Wrote location list to file");
		else
			System.out.println("Error writing location list to file");
	}
	
	public void stop(){
		try{
			//Try local osgi call to pocketdiscoveryService
			if(discoveryService!=null){
				discoveryService.removeService("descriptionUrl:"+"http://"+InetAddress.getLocalHost().getHostAddress()+":8080/axis/services/locationService?wsdl");
			}
			//Try webservice call to discoveryService
			else{
				discoveryServiceProxy.removeService("descriptionUrl:"+"http://"+InetAddress.getLocalHost().getHostAddress()+":8080/axis/services/locationService?wsdl");
			}
		}
		catch(Exception e){
			System.out.println("<LocationService>Unable to unregister myself from discoveryservice");
		}
	}
	
}
