package no.ubicollab.osgi.service.location;

/**
 * Interface for the LocationService<br>
 * The location service handles mappings from positions (coordinates) to locations. E.g. It-building, Manhattan, Europe etc..
 *  
 * @author Andreas Brustad
 *
 */
public interface LocationService {
	
	/**
	*Adds a new location to the service's list of locations. Overrides the old location if locationName already exists. <br>
	*The location has the shape of a polygon. The first latitude and the first longitude in the lists define the first point of the polygon etc. The two lists must therefore have the same length
	*@param locationName Name of the new location
	*@param longitudes List of longitudes (x-axis) expressed in decimal degrees. Max precision is 6 decimals. Use negative values for west
	*@param latitudes List of latitudes (y-axis) expressed in decimal degrees. Max precision is 6 decimals. Use negative values on the southern hemisphere
	*@return true
	*@see java.awt.Polygon
	*/
	public boolean addLocation(String locationName, double longitudes[], double[] latitudes, int altitude);
	
	/**
	 * Removes a location from the service's list of locations. Does nothing if the locationName is not found
	 * @param locationName Name of the location to remove
	 * @return true
	 */
	public boolean removeLocatoin(String locationName);
	
	/**
	 * Returns the name of the location containing the position specified by the parameters.<br>
	 * The first maching location is returned if the position is found in several locations.
	 * @param longitude Longitude of the position expressed in decimal degrees. Use negative values for west
	 * @param latitude Latitude of the position expressed in decimal degrees. Use negative values for south
	 * @param altitude Altitude of the position
	 * @param altitudeErrorMargin Allowed altitude error. <br>Location is returned if Math.abs(correctAltitude-altitude)<=altitudeErrorMargin;
	 * @return Name of the location containing the spesified position, or null if no location contains the position.<br> The first maching location is returned if the position is found in several locations.
	 */
	public String getLocation(double longitude, double latitude, int altitude, int altitudeErrorMargin);
	
	/**
	 * Returns the name of the location containing the position specified by the parameters, without considering altitude.<br>
	 * The first maching location is returned if the position is found in several locations.
	 * @param longitude Longitude of the position expressed in decimal degrees. Use negative values for west
	 * @param latitude Latitude of the position expressed in decimal degrees. Use negative values for south
	 * @return Name of the location containing the spesified position, or null if no location contains the position.<br> The first maching location is returned if the position is found in several locations.
	 */
	public String getLocation(double longitude, double latitude);
	
}
