package no.ubicollab.osgi;

import java.net.MalformedURLException;
import java.rmi.RemoteException;

import javax.xml.rpc.ServiceException;

import no.ubicollab.osgi.service.pocketdiscovery.PocketDiscoveryService;
import no.ubicollab.axis.services.discoveryService.*;



public class PocketDiscoveryImpl implements PocketDiscoveryService {
	
	private DiscoveryImpl webServiceProxy;
	private FileAccess fileAccess;
	
	public PocketDiscoveryImpl(){
		
		//create the proxy to the webservice that calls are forwarded to
		DiscoveryImplServiceLocator service = new DiscoveryImplServiceLocator();
		fileAccess = new FileAccess();
		try {
			String url = fileAccess.readLineAndClose();
			if(url!=null)
				webServiceProxy = service.getdiscoveryService(new java.net.URL(url));
			else{
				System.out.println("Could not find textfile. Using http://gakkan.no:8080/discoveryService as default discoveryService location");
				webServiceProxy = service.getdiscoveryService(new java.net.URL("http://gakkan.no:8080/discoveryService"));
			}
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (ServiceException e) {
			e.printStackTrace();
		}
		
	}
	
	public boolean registerDiscoveryService(String url){
//		update the proxy to the webservice that calls are forwarded to
		DiscoveryImplServiceLocator service = new DiscoveryImplServiceLocator();
		try {
			webServiceProxy = service.getdiscoveryService(new java.net.URL(url));
			//Write url to file
			if(!fileAccess.writeLineAndClose(url))
				System.out.println("Could not write url to file");
			System.out.println("<PocketDiscovery>Registred new discoveryservice on "+url);
			return true;
		} catch (MalformedURLException e) {
			e.printStackTrace();
			return false;
		} catch (ServiceException e) {
			e.printStackTrace();
			return false;
		}
	}
	
	public boolean registerService(String tagId, String url) {
		try {
			System.out.println("<PocketDiscovery>Trying to register new service on "+url);
			return webServiceProxy.registerService(tagId, url);
		} catch (RemoteException e) {
			System.out.println("<PocketDiscovery> Could not register Service. RemoteException when trying to contact discService");
			//e.printStackTrace();
			return false;
		}
		catch(Exception generalException){
			System.out.println("<PocketDiscovery> Could not register Service. Exception when trying to contact discService");
			//generalException.printStackTrace();
			return false;
		}
	}

	public boolean registerService(String url) {
		try {
			System.out.println("<PocketDiscovery>Trying to register new service on "+url);
			return webServiceProxy.registerService(url);
		} catch (RemoteException e) {
			System.out.println("<PocketDiscovery> Could not register Service. RemoteException when trying to contact discService");
			//e.printStackTrace();
			return false;
		}
		catch(Exception generalException){
			System.out.println("<PocketDiscovery> Could not register Service. Exception when trying to contact discService");
			//generalException.printStackTrace();
			return false;
		}
	}

	public boolean registerService(String tagId, String uuId, String name, String type, String protocol, String descriptionUrl, String serviceUrl, String owner) {
		try {
			System.out.println("<PocketDiscovery>Trying to register new service on "+descriptionUrl);
			return webServiceProxy.registerService(tagId, uuId, name, type, protocol, descriptionUrl, serviceUrl, owner);
		} catch (RemoteException e) {
			System.out.println("<PocketDiscovery> Could not register Service. RemoteException when trying to contact discService");
			//e.printStackTrace();
			return false;
		}
		catch(Exception generalException){
			System.out.println("<PocketDiscovery> Could not register Service. Exception when trying to contact discService");
			//generalException.printStackTrace();
			return false;
		} 
	}

	public String getService(String searchString) {
		try {
			System.out.println("<PocketDiscovery>Trying to get service on with searchString "+searchString);
			return webServiceProxy.getService(searchString);
		} catch (RemoteException e) {
			System.out.println("<PocketDiscovery> Could not get Service. RemoteException when trying to contact discService");
			//e.printStackTrace();
			return null;
		}
		catch(Exception generalException){
			System.out.println("<PocketDiscovery> Could not get Service. Exception when trying to contact discService");
			//generalException.printStackTrace();
			return null;
		}
	}

	public boolean removeService(String removeString) {
		try {
			System.out.println("<PocketDiscovery>Trying to remove service with string "+removeString);
			return webServiceProxy.removeService(removeString);
		} catch (RemoteException e) {
			System.out.println("<PocketDiscovery> Could not remove Service. RemoteException when trying to contact discService");
			//e.printStackTrace();
			return false;
		}
		catch(Exception generalException){
			System.out.println("<PocketDiscovery> Could not remove Service. Exception when trying to contact discService");
			//generalException.printStackTrace();
			return false;
		}
	}
	
	public String[] getInfoAboutService(String url){
		try {
			System.out.println("<PocketDiscovery>Trying to get infor aboute service with url "+url);
			return webServiceProxy.getInfoAboutService(url);
		} catch (RemoteException e) {
			System.out.println("<PocketDiscovery> Could not get info about ser Service. RemoteException when trying to contact discService");
			//e.printStackTrace();
			return null;
		}
		catch(Exception generalException){
			System.out.println("<PocketDiscovery> Could not get info about Service. Exception when trying to contact discService");
			//generalException.printStackTrace();
			return null;
		}
	}	
}
