package no.ubicollab.osgi;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.Timer;
import java.util.TimerTask;

import no.ubicollab.osgi.service.pocketdiscovery.PocketDiscoveryService;
import no.ubicollab.osgi.service.positioning.PositioningService;


public class PositioningImpl extends TimerTask implements PositioningService {
	
	private PocketDiscoveryService discovery;
	private DynamicProxyGenerator dynProxyGen;
	
	private ArrayList gpsPlugins = new ArrayList();
	private ArrayList gsmPlugins = new ArrayList();
	private ArrayList otherPlugins = new ArrayList();
	
	//private ConfigFileManager configFileManager;
	
	//Start PositioningImpl registering it in the local discovery service using a osgi call
	public PositioningImpl(PocketDiscoveryService discoveryService){
		discovery=discoveryService;
		dynProxyGen = new DynamicProxyGenerator();
		Timer updateRefTimer = new Timer();
		updateRefTimer.scheduleAtFixedRate(this,1000*60*35,1000*60*35);
		
		/*configFileManager = new ConfigFileManager();
		String[] config = configFileManager.readConfigFile();
		if(config==null){
			System.out.println("GpsPositioningImpl: There was an error reading the config file. Stopping...");
			activator.stop(Activator.bc);
		}
		else{
			ownerUsername= config[0];
			gpsComPort= config[1];
			discoveryServiceUrl = config[2];
		*/
		try {
			//Register this service in the discoveryService
			String serviceUrl="http://"+InetAddress.getLocalHost().getHostAddress()+":8080/axis/services/positioningService";
			String descriptionUrl=serviceUrl+"?wsdl";
			discoveryService.registerService("","","PositioningService","no.ubicollab.osgi.service.positioning","ws",descriptionUrl,serviceUrl,"");
			
		} 
		catch (UnknownHostException e) {
			System.out.println("<PositioningService>Error registering service in pocketDiscovery");
			e.printStackTrace();
		}
		
		System.out.println("<PositioningService>Updating pluginreferences...");
		updatePluginReferences();
		System.out.println("<PositioningService>...done");
		
		/*For testing only
		double[] pos = getPosition("andreabr");
		if(pos!=null){
        	System.out.println("andreabr: long:"+pos[0]+"lat:"+pos[1]+"alt:"+pos[2]);
        }
        else
        	System.out.println("No pos for andreabr:null");
        
		double[] pos2 = getPosition("mosveen");
		if(pos2!=null){
        	System.out.println("mosveen: long:"+pos2[0]+"lat:"+pos2[1]+"alt:"+pos2[2]);
        }
        else
        	System.out.println("No pos for mosveen:null");
        
		double[] pos3 = getPosition("julenissen");
		if(pos3!=null){
        	System.out.println("julenissen: long:"+pos3[0]+"lat:"+pos3[1]+"alt:"+pos3[2]);
        }
        else
        	System.out.println("No pos for julenissen:null");
		*/
		//}
	}

	public double[] getPosition(String username) {
		//Try gps first. If not found, try gsm. If still not found, try other plugins.
		for(int i=0; i<gpsPlugins.size(); i++){
			Plugin gpsPlugin= (Plugin)gpsPlugins.get(i);
			//invoke on this plugin
			double[] pos = gpsPlugin.getPosition(username);
			if(pos!=null){
				System.out.println("<PositioningService>Returning position from gps-plugin nr:"+i);
				return pos;
			}
		}
		
		for(int j=0; j<gsmPlugins.size(); j++){
			Plugin gsmPlugin= (Plugin)gsmPlugins.get(j);
			//invoke on this plugin
			double[] pos = gsmPlugin.getPosition(username);
			if(pos!=null){
				System.out.println("<PositioningService>Returning position from gsm-plugin nr:"+j);
				return pos;
			}
		}
		
		for(int k=0; k<otherPlugins.size(); k++){
			Plugin otherPlugin= (Plugin)otherPlugins.get(k);
			//invoke on this plugin
			double[] pos = otherPlugin.getPosition(username);
			if(pos!=null){
				System.out.println("<PositioningService>Returning position from other-plugin nr:"+k);
				return pos;
			}
		}
		System.out.println("<PositioningService>No position found in any plugins for user"+username);
		return null;
	}
	
	public void updatePluginReferences(){
		
		gpsPlugins.clear();
		gsmPlugins.clear();
		otherPlugins.clear();
		
		String[] pluginServiceUrls = searchForPlugins();
		Plugin[] plugins = createPluginsFromUrls(pluginServiceUrls);
		
		for(int i=0; i < plugins.length; i++){
			if("no.ubicollab.osgi.service.positioning.gps".equals(plugins[i].type)){
				gpsPlugins.add(plugins[i]);
				System.out.println("<PositioningService>Added a gps plugin to the list of plugins");
			}
			else if("no.ubicollab.osgi.service.positioning.gsm".equals(plugins[i].type)){
				gsmPlugins.add(plugins[i]);
				System.out.println("<PositioningService>Added a gsm plugin to the list of plugins");
			}
			else{
				gsmPlugins.add(plugins[i]);
				System.out.println("<PositioningService>Added a plugin of unknown type to the list of plugins");
			}
		}	
	}
	
	public void pocketDiscoveryStopped(){
		discovery=null;
	}
	
	public void stop(){
		try{
			if(discovery!=null){
				discovery.removeService("descriptionUrl:"+"http://"+InetAddress.getLocalHost().getHostAddress()+":8080/axis/services/positioningService?wsdl");
			}
		}
		catch(Exception e){
			System.out.println("<PositioningService>Unable to unregister myself from discoveryservice");
		}
	}
	
	private String[] searchForPlugins(){
		try{
			if(discovery==null){
				System.out.println("<PositioningService>Can not updateReferences. No contact with discoveryService");
				return new String[] {};
			}
			String pluginUrls = discovery.getService("type:no.ubicollab.osgi.service.positioning.* protocol:ws");
			
			//pluginUrls is of format "url ws url ws url ws..."
			
			if ("No matching services".equals(pluginUrls) || "Error: Unknown service protocol".equals(pluginUrls) || pluginUrls == null || pluginUrls.length() <= 0)
			{
				System.out.println("<PositioningService>No plugins found");
				return new String[] { };
			}
			ArrayList urls = new ArrayList();
	
			//Copy urls only to Arraylist
			StringTokenizer st = new StringTokenizer(pluginUrls);
			int i = 0;
			while (st.hasMoreTokens())
			{
				String token = st.nextToken();
				//System.out.println("Found:" + token + " i:" + i);
				if (i == 0)
					urls.add(token);
	
				i = (i + 1) % 2;
			}
			//Copy from ArrayList to String[]
			String[] urlArray = new String[urls.size()];
			for (int j = 0; j < urls.size(); j++)
			{
				urlArray[j] = (String)urls.get(j);
			}
			return urlArray;
		}
		catch (Exception e) {
			System.out.println("<PositioningService>Can not updateReferences. Error contacting discoveryService");
			return new String[] {};
		}
	}
	
	private Plugin[] createPluginsFromUrls(String[] serviceUrls){
		ArrayList plugins = new ArrayList();
		try{
			for(int i=0; i<serviceUrls.length; i++){
				String[] serviceInfo = discovery.getInfoAboutService(serviceUrls[i]);
				if(serviceInfo!=null && serviceInfo.length>=10){
					String descriptionUrl = serviceInfo[6];
					String type = serviceInfo[4];
					if(descriptionUrl!=null && descriptionUrl.length()>0 && type!=null && type.length()>0){
						PositioningPluginService proxy = dynProxyGen.getProxy(descriptionUrl);
						//PositioningPluginService proxy = dynProxyGen.getProxy("http://localhost:8080/axis/services/gpsPositioningService?wsdl");
						//PositioningPluginService proxy = dynProxyGen.getProxy("http://folk.ntnu.no/matsjorg/");
						if(proxy != null){
							plugins.add(new Plugin(descriptionUrl,type,proxy));
							System.out.println("<PositioningService>Proxy successfully generated from descurl: "+descriptionUrl+" with type: "+type);
						}
					}
				}
			}
		}
		catch (Exception e) {
			System.out.println("<PositioningService>Error crating pluginreferences");
		}
		//Copy from ArrayList to Plugin[]
		Plugin[] pluginArray = new Plugin[plugins.size()];
		for (int j = 0; j < plugins.size(); j++)
		{
			pluginArray[j] = (Plugin)plugins.get(j);
		}
		return pluginArray;
	}
	
	public void run(){
		System.out.println("<PositioningService>Starting scheduled plug-in reference update...");
		updatePluginReferences();
		System.out.println("<PositioningService>...ended update of plug-ins");
	}
}
