package no.ubicollab.osgi;

import java.net.InetAddress;
import java.util.*;
import org.osgi.framework.*;

import no.ubicollab.osgi.service.pocketdiscovery.PocketDiscoveryService;
import no.ubicollab.osgi.service.slideshow.*;

public class Activator implements BundleActivator {
	
	private PocketDiscoveryService discoveryService;
	private String descriptionUrl;
    
    public void start(BundleContext context) throws BundleException {
    	try {
	        Properties properties = new Properties();
	        properties.put("Language", "English");
	        properties.put("SOAP.service.name", "slideshowService");
	        
	        ServiceReference reference = context.getServiceReference(PocketDiscoveryService.class.getName());
	        if(reference != null){
	        	//Local pocket discovery service exists.
	        	//Start DataStorageService registering it in this discovery service using a local osgi call
	        	discoveryService = (PocketDiscoveryService)context.getService(reference);
	        	//gps=new GsmPositioningImpl(discoveryService, this);
	        	String serviceUrl = "http://" + InetAddress.getLocalHost().getHostAddress() + ":8080/axis/services/slideshowService";
				descriptionUrl = serviceUrl + "?wsdl";
	        	discoveryService.registerService("", "", "slideShowService", "no.ubicollab.osgi.service.slideshow", "ws", descriptionUrl, serviceUrl, "");
	        	context.ungetService(reference);
	        }
	        else{
	        	//No local pocket discovery service exists
	        	//Start GpsPositioningImpl registering it in an external discoveryservice using a webservice call.
	        	//The url to the external discoveryserivce is retrieved from the configfile
	        	//gps=new GsmPositioningImpl(this);
	        }
	        
	        context.registerService(SlideshowService.class.getName(), new SlideshowImpl(discoveryService), properties);
    	} catch (Exception e) {
    		throw new BundleException("Failed to start Slideshow Service", e);
    	}
    }
    
    public void stop(BundleContext context) throws BundleException {
    	try {
    		discoveryService.removeService("descriptionUrl:" + descriptionUrl);
    		context = null;
    	} catch (Exception e) {
    		throw new BundleException("Failed to stop Slideshow Service", e);
    	}
    }
    
}
