using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Intel.DeviceBuilder;
using Intel.UPNP;
using System.Threading;
using CFLightControl.PocketDiscoveryService;

namespace CFLightControl
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            

        }
 
        private String[] searchForLights()
        {
            try
            {
                PocketDiscoveryImplService discoveryService = new PocketDiscoveryImplService();
                searchButton.Text = "Searching...";
                searchButton.Enabled = false;
                //searchResult should have the format: url protocol url protocol.....
                string searchResult = discoveryService.getService("type:urn:schemas-upnp-org:device:binarylight:1 protocol:upnp");
                searchButton.Text = "Search";
                searchButton.Enabled = true;
                if ("No matching services".Equals(searchResult) || "Error: Unknown service protocol".Equals(searchResult) || searchResult == null || searchResult.Length <= 0)
                {
                    return new string[] { };
                }
                else
                {
                    //Copy urls only
                    string[] urlAndProtocol = searchResult.Split(new Char[] { ' ' });
                    string[] urls = new string[urlAndProtocol.Length / 2];
                    for (int i = 0; i < urlAndProtocol.Length; i += 2)
                    {
                        urls[i / 2] = urlAndProtocol[i];
                    }
                    return urls;
                }
            }
            catch (Exception e)
            {
                MessageBox.Show("Could not search for lights");
                MessageBox.Show(e.Message);
                MessageBox.Show(e.StackTrace);
                searchButton.Text = "Search";
                searchButton.Enabled = true;
                return new string[] { };
            }
        }

        private void AddDevice(UPnPDeviceFactory sender, UPnPDevice device, Uri URL)
        {
            //Make sure the device has a switchPowerService
            if (device.GetServices(CpSwitchPower.SERVICE_NAME).Length >= 1)
            {
                deviceList.Invoke(new myMethodDelegate(ThreadAddDevice),new UPnPDevice[] {device});
            }
        }


        private void FailedDevice(UPnPDeviceFactory sender, Uri URL, Exception e)
        {
            //MessageBox.Show("Device creation failed with message: " + e.Message);
        }

        private void searchButton_Click(object sender, EventArgs e)
        {
            deviceList.Items.Clear();
            
            string[] descUrls = searchForLights();

            foreach (string url in descUrls)
            {
                new UPnPDeviceFactory(new Uri(url), 10, new UPnPDeviceFactory.UPnPDeviceHandler(AddDevice), new UPnPDeviceFactory.UPnPDeviceFailedHandler(FailedDevice));
            }
            
            //UPnPDeviceFactory devFact = new UPnPDeviceFactory(new Uri("http://129.241.103.124:80/description.xml"), 10, new UPnPDeviceFactory.UPnPDeviceHandler(AddDevice), new UPnPDeviceFactory.UPnPDeviceFailedHandler(FailedDevice));
            //UPnPDeviceFactory devFact2 = new UPnPDeviceFactory(new Uri("http://129.241.103.124:8080/siemensTestDevice/description.xml"), 10, new UPnPDeviceFactory.UPnPDeviceHandler(AddDevice), new UPnPDeviceFactory.UPnPDeviceFailedHandler(FailedDevice));
        }

        private void onButton_Click(object sender, EventArgs e)
        {
            try
            {
                CpSwitchPower proxy = (CpSwitchPower)deviceList.SelectedItem;
                proxy.Sync_SetTarget(true);
                status.Text = "Status: ON";
            }
            catch (Exception) { }
        }

        private void offButton_Click(object sender, EventArgs e)
        {
            try
            {
                CpSwitchPower proxy = (CpSwitchPower)deviceList.SelectedItem;
                proxy.Sync_SetTarget(false);
                status.Text = "Status: OFF";
            }
            catch (Exception) { }
        }

        private void statusButton_Click(object sender, EventArgs e)
        {
            UpdateStatus();
        }

        private void ThreadAddDevice(UPnPDevice device)
        {
            CpSwitchPower SwitchPower = new CpSwitchPower(device.GetServices(CpSwitchPower.SERVICE_NAME)[0]);
            deviceList.Items.Add(SwitchPower);
            //If this is the first device. Select it
            if (deviceList.Items.Count == 1)
            {
                deviceList.SelectedItem = SwitchPower;
            }
        }

        private void deviceList_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateStatus();
        }

        private void UpdateStatus()
        {
            CpSwitchPower proxy;
            try
            {
                proxy = (CpSwitchPower)deviceList.SelectedItem;
                //MessageBox.Show("Selected index changed "+proxy);
                bool result = proxy.Sync_GetStatus();
                if (result)
                    status.Text = "Status: ON";
                else
                    status.Text = "Status: OFF";
            }
            catch (Exception ex)
            {
                try
                {
                    //MessageBox.Show("Funka ikke prover plan B :" + ex.Message);
                    Boolean res = false;
                    proxy = (CpSwitchPower)deviceList.SelectedItem;
                    proxy.Sync_GetStatus(out res);
                    if (res)
                        status.Text = "Status: ON";
                    else
                        status.Text = "Status: OFF";
                }
                catch (Exception ex2)
                {
                    MessageBox.Show("Error getting light status" + ex2.Message);
                }
            }
        }
    }
    public delegate void myMethodDelegate(UPnPDevice device);
}