using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using DataStorageServiceTester.DataStorageService;
using System.Web.Services.Protocols;

namespace DataStorageServiceTester
{
    public partial class Form1 : Form
    {
        private const string appname = "DataStorage";
        private const string username = "andreabr";

        private DataStorageImplService DataStorageService;
        public Form1()
        {
            InitializeComponent();
            DataStorageService = new DataStorageImplService();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            bool success;
            try
            {
                byte[] bytes = FileAccess.ReadFile(textBox1.Text);
                int i = textBox1.Text.LastIndexOf(@"\");
                if (i == -1)
                    success = DataStorageService.storeData(username, appname, textBox1.Text, bytes);
                else
                    success = DataStorageService.storeData(username, appname, textBox1.Text.Substring(i + 1), bytes);
            }
            catch (SoapException se)
            {
                MessageBox.Show(se.Message);
                MessageBox.Show(se.StackTrace);
                success = false;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                MessageBox.Show(ex.StackTrace);
                success = false;
            }
            MessageBox.Show("Success: " + success);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            try
            {
                byte[] data = DataStorageService.retrieveData(username, appname, textBox2.Text);
                if (data != null && data.Length > 0)
                {
                    saveData(textBox2.Text, data);
                }
            }
            catch (SoapException)
            {
                MessageBox.Show("error retrieving file");
            }
        }

        private void browseButton_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog1 = new OpenFileDialog();
            openFileDialog1.Filter = "All files (*.*)|*.*";
            openFileDialog1.RestoreDirectory = true;
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                textBox1.Text = openFileDialog1.FileName;
            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            refreshFileList();
        }

        private void button4_Click(object sender, EventArgs e)
        {
            string selected = (string)listBox1.SelectedItem;
            if (selected != null)
            {
                byte[] data=null;
                try
                {
                    data = DataStorageService.retrieveData(username, appname, selected);
                    if (data != null && data.Length > 0)
                    {
                        saveData(selected, data);
                    }
                }
                catch (System.IO.EndOfStreamException)
                {
                    Console.WriteLine("End of stream exception Trying to store anyway");
                    saveData(selected, data);
                }
            }
        }

        private void button5_Click(object sender, EventArgs e)
        {
            string selected = (string)listBox1.SelectedItem;
            
            if (selected != null)
            {
                bool success = DataStorageService.deleteData(username, appname, selected);
                refreshFileList();
                //MessageBox.Show("Success: " + success);
            }
            
        }

        private void refreshFileList()
        {
            string[] filenames = DataStorageService.listFileNames(username, appname);
            if (filenames != null)
            {
                listBox1.Items.Clear();
                foreach (string file in filenames)
                {
                    listBox1.Items.Add(file);
                }
            }
        }

        private void saveData(string file, byte[] data)
        {
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.Filter = "All files (*.*)|*.*";
            saveFileDialog1.RestoreDirectory = true;
            saveFileDialog1.FileName = @"E:\" + file;
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                FileAccess.WriteFile(saveFileDialog1.FileName, data);
            }
        }
    }
}