using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Locator.PocketDiscoveryService;
using Locator.LocationService;
using Locator.PositioningService;
using Locator.CollaborationService;


namespace Locator
{
    public partial class Form1 : Form
    {
        private PocketDiscoveryImplService pocketdiscoveryService;
        private PositioningImplService positioningService;
        private LocationImplService locationService;
        private CollaborationImplService collaborationService;

        private DataTable dataTable;

        public Form1()
        {
            InitializeComponent();
            try
            {
                pocketdiscoveryService = new PocketDiscoveryImplService();
                
                string[] positioningUrls = searchForServices("type:no.ubicollab.osgi.service.positioning protocol:ws");
                string[] locationUrls = searchForServices("type:no.ubicollab.osgi.service.location protocol:ws");
                if (positioningUrls.Length < 1 || locationUrls.Length < 1)
                    MessageBox.Show("Could not find position service and/or location service");
                else
                {
                    //use the first avilable position and location service
                    positioningService = new PositioningImplService();
                    locationService = new LocationImplService();

                    positioningService.Url = positioningUrls[0];
                    locationService.Url = locationUrls[0];

                    //No need to search for collaborationService. It is running on a server with a well known ip-address.
                    collaborationService = new CollaborationImplService();
                    
                    
                    dataTable = CreateDataTable();
                    dataGrid1.DataSource = dataTable;

                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Cannot start :" + ex.Message);
            }
        }

        public DataTable CreateDataTable()
        {
            DataTable table = new DataTable("Users");

            DataColumn column1 = new DataColumn("Collab.ins", Type.GetType("System.String"));
            column1.DefaultValue = "<Unknown>";
            DataColumn column2 = new DataColumn("Username", Type.GetType("System.String"));
            column2.DefaultValue = "<Unknown>";
            DataColumn column3 = new DataColumn("Position", Type.GetType("System.String"));
            column3.DefaultValue = "<Unknown>";
            DataColumn column4 = new DataColumn("Location", Type.GetType("System.String"));
            column4.DefaultValue = "<Unknown>";
            table.Columns.Add(column1);
            table.Columns.Add(column2);
            table.Columns.Add(column3);
            table.Columns.Add(column4);
            return table;
        }

        private String[] searchForServices(string searchParameter)
        {
            try
            {
                //searchResult should have the format: url protocol url protocol.....
                string searchResult = pocketdiscoveryService.getService(searchParameter);
                if ("No matching services".Equals(searchResult) || "Error: Unknown service protocol".Equals(searchResult) || searchResult == null || searchResult.Length <= 0)
                {
                    return new string[] { };
                }
                else
                {
                    //Copy urls only
                    string[] urlAndProtocol = searchResult.Split(new Char[] { ' ' });
                    string[] urls = new string[urlAndProtocol.Length / 2];
                    for (int i = 0; i < urlAndProtocol.Length; i += 2)
                    {
                        urls[i / 2] = urlAndProtocol[i];
                    }
                    return urls;
                }
            }
            catch (Exception e)
            {
                MessageBox.Show("Could not search for services");
                MessageBox.Show(e.Message);
                MessageBox.Show(e.StackTrace);
                return new string[] { };
            }
        }

        private void PopulateDataTable()
        {
            try
            {
                collaborationService.Url = collabUrlBox.Text;
                string[] collabIDs = collaborationService.getPersonsCollaborationInstances(usernameBox.Text);
                foreach (string id in collabIDs)
                {
                    string collabName = collaborationService.getCollabInstName(id);
                    string[] users = collaborationService.getAllPersonsInCollabInst(id);
                    foreach (string user in users)
                    {
                        DataRow row = dataTable.NewRow();
                        row[0] = collabName;
                        row[1] = user;
                        dataTable.Rows.Add(row);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Unable to populate table " + ex.Message);
            }
        }

        private void updateButton_Click(object sender, EventArgs e)
        {
            try
            {
                foreach (DataRow row in dataTable.Rows)
                {
                    double[] pos = positioningService.getPosition((string)row["Username"]);
                    if (pos != null && pos.Length >= 3)
                    {
                        row["Position"] = pos[0] + ";" + pos[1] + ";" + pos[2];
                        string location = locationService.getLocation(pos[0], pos[1]);
                        if (location != null)
                        {
                            row["Location"] = location;
                        }
                        row.AcceptChanges();
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Unable to update locations " + ex.Message);
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            dataTable.Rows.Clear();
            PopulateDataTable();
        }
    }
}