using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using Cathexis.IDBlue;
using System.Windows.Forms;
using RFID_registry_app.DiscoveryService;
using System.Web.Services.Protocols;

namespace RFID_registry_app
{
    class RFID_scanner
    {
        private Form1 mother;
        private Cathexis.IDBlue.IDBlueNET _driver;

        //Create new fileAccess with on default filename
        //private FileAccess fileAccess;

        public RFID_scanner(Form1 mother/*, FileAccess fileAccess*/)
        {
            this.mother = mother;
            //this.fileAccess = fileAccess;
            LoadDrivers();
        }

        public bool LoadDrivers()
        {
            _driver = new Cathexis.IDBlue.IDBlueNET(
               Cathexis.IDBlue.IDBlueConnectionMethod.Franson);

            // _driver = new Cathexis.IDBlue.IDBlueNET(
            //     Cathexis.IDBlue.IDBlueConnectionMethod.Baracoda);

            // Device event handlers
            _driver.OnConnectFailed += new Cathexis.IDBlue.DeviceEvent(_driver_OnConnectFailed);
            _driver.OnDeviceActivated += new Cathexis.IDBlue.DeviceActivated(_driver_OnDeviceActivated);
            _driver.OnDeviceDeactivated += new Cathexis.IDBlue.DeviceDeactivated(_driver_OnDeviceDeactivated);


            // Tag event handlers
            _driver.OnAsyncButtonPress += new Cathexis.IDBlue.ButtonPressed(_driver_OnAsyncButtonPress);
            _driver.OnAsyncTagID += new Cathexis.IDBlue.AsyncTagID(_driver_OnAsyncTagID);

            return true;
        }

        //Connects or disconnects the reader
        //returns true if the new status of the connection is "connected" false if the device was disconnected
        public bool Connect()
        {
            if (_driver.IsConnected)
            {
                //Append("Disconnecting from device " + _driver.Device);
                _driver.Disconnect();

                return false;
            }
            else
            {
                // Set new device address
                //_driver.Device = devAddress;
                _driver.Device = "00:A0:96:0B:D5:71";

                //Append("Connecting to device: " + _driver.Device);

                // Initiate a connection
                Cathexis.IDBlue.ErrorCode err = _driver.BeginConnection();
                if (err != Cathexis.IDBlue.ErrorCode.Success)
                {
                    MessageBox.Show("Error: could not initiate connection to device");
                    return false;
                }

                return true;
            }
        }

        public void DisconnectAndClose()
        {
            try
            {
                _driver.Disconnect();
                _driver.Dispose();
            }
            catch (Exception)
            {
            }
        }

        public void SetInDataMode()
        {
            _driver.ContinuousScanEnabled = false;
            _driver.SetConnectedMode(Cathexis.IDBlue.ConnectedOpMode.REACTIVE);
        }

        public void SetInIdMode()
        {
            _driver.ContinuousScanEnabled = false;
            _driver.SetConnectedMode(Cathexis.IDBlue.ConnectedOpMode.TAGID);
        }


        #region "IDBlue Event Handlers"
      
        void _driver_OnConnectFailed()
        {
            //Append("Connect Failed");
        }

        void _driver_OnDeviceActivated()
        {
            //this.AddMessage("Connected established");

            this._driver.Timestamp = DateTime.Now;
            SetInDataMode();
            //this.AddMessage("Timestamp updated to " + DateTime.Now.ToShortTimeString());
            //Append("Device activated");
        }

        void _driver_OnDeviceDeactivated()
        {
            //this.AddMessage("Connection lost");
            //this.btnConnect.Text = "Connect";
            //Append("Device deactivated");
        }


        void _driver_OnAsyncButtonPress()
        {

            Cathexis.IDBlue.SelectMediaObject tagObj;
            Cathexis.IDBlue.ErrorCode err;

            _driver.BeginCommands();

            // Attempt to scan an RFID tag
            err = _driver.ScanMediaId(out tagObj);
            if (err != ErrorCode.Success)
            {
                _driver.EndCommands(true);
                return;
            }

            // Read block data from this tag - note the use of passing
            // in the Media ID (tag ID) obtained from the ScanMediaId
            // method


            ArrayList masterData = new ArrayList();
            ArrayList data = new ArrayList();
            //Read all 128 bytes (32 blocks)
            int i = 0;
            while (i < 32)
            {
                data = new ArrayList();
                err = _driver.ReadBlocks(tagObj.MediaID, i, 16, out data);

                if (err != ErrorCode.Success)
                {
                    _driver.EndCommands(true);
                    //Append("Error reading tag! i=" + i);
                    return;
                }
                masterData.Add(data);
                i += 16;
                //Append("Read 16 blocks of data");
            }

            // Successful end of RFID operations
            _driver.EndCommands(false);

            // Process the data in the data arraylist
            // Note that masterData is an ArrayList containing Arraylists
            String url = "";
            foreach (ArrayList dataList in masterData)
            {
                url += Utils.GetStringFromArrayList(dataList);
            }

            string tagId=BitConverter.ToString(tagObj.MediaID);
            int index=-1;
            while ((index = tagId.IndexOf('-')) != -1)
            {
                tagId=tagId.Remove(index, 1);
            }

            //Register this tagID and URL as a service in the serviceDiscovery
            if (mother.registerServiceChoice.Checked)
            {
                DialogResult dialogRes = MessageBox.Show("Add service with tagID: " + tagId + " and url: " + url + " to discovery service?", "Add service?", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1);
                if (dialogRes == DialogResult.Yes)
                {

                    //TODO
                    //Call WS here with tagId and url

                    try
                    {
                        PocketDiscoveryImplService discoveryService = new PocketDiscoveryImplService();
                        discoveryService.Url = mother.DiscServiceUrl;
                        bool success = discoveryService.registerService(tagId, url);
                        if (success)
                            MessageBox.Show("Added to discoveryservice");
                        else
                            MessageBox.Show("Error registering service");
                    }
                    catch (SoapException)
                    {
                        MessageBox.Show("Error registrating service");
                    }
                    catch (Exception)
                    {
                        MessageBox.Show("Unknown Exception");
                    }
                }
            }
            //Set the url from the tag as default discoveryService address
            else if (mother.discoveryServiceChoice.Checked)
            {
                DialogResult dialogRes = MessageBox.Show("Set url: " + url + " as default discovery service address?", "Discovery Service address", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1);
                if (dialogRes == DialogResult.Yes)
                {
                    /*FileAccess not in use anymore. Send url to the discoveryService on 127.0.0.1
                    if (fileAccess.WriteLineAndClose(url))
                    {
                        mother.DiscServiceUrl = url;
                        MessageBox.Show("DiscoveryService url updated and written to file!");
                    }
                    else
                        MessageBox.Show("Error updating discoveryService url");
                    */
                    try
                    {
                        PocketDiscoveryImplService discoveryService = new PocketDiscoveryImplService();
                        discoveryService.Url = mother.DiscServiceUrl;
                        bool success = discoveryService.registerDiscoveryService(url);
                        if (success)
                            MessageBox.Show("DirectoryService address updated");
                        else
                            MessageBox.Show("Error updating discoveryService address");
                    }
                    catch (SoapException)
                    {
                        MessageBox.Show("Error updating discoveryService address. SoapException");
                    }
                    catch (Exception)
                    {
                        MessageBox.Show("Unknown Exception");
                    }
                    
                }
            }

        }
            

        /// <summary>
        /// TODO: describe usage of callback
        /// </summary>
        void _driver_OnAsyncTagID()
        {
            string tagId = _driver.AsyncData.TagId;
            //Append("Received async tag ID: " + tagId);
        }

        #endregion

    }
}
