using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using Cathexis.IDBlue;

namespace RFID_registry_app
{
    class Utils
    {
        public static string GetStringFromArrayList(ArrayList data)
        {
            string asString = "";
            ASCIIEncoding ascii = new ASCIIEncoding();
            foreach (Block block in data)
            {
                asString = asString + ascii.GetString(block.Data, 0, block.Data.Length);
            }
            //remove 0x00 at the end
            if (asString.IndexOf((char)0x00) != -1)
                return asString.Substring(0, asString.IndexOf((char)0x00));
            else
                return asString;
        }


        /*
        This method returns an ArrayList of ArrayLists
        The inner ArrayLists contains <nrOfBlocksPrList> Blocks (except for the last element, which contains "the rest") which can be used as input to the readBlocks and WriteBlocks-methods 
        12 or 16 blocks is max recommended nr of blocks pr write or read in the readBlocks and WriteBlocks methods

        example usage
        int i=0;
        foreach(ArrayList writeData in SplitAndCreateArrayList("some text to be written", 12))
        {
            _driver.WriteBlocks(tagObj.MediaID, i, writeData);
            i += writeData.Count;
        }
        */
        public static ArrayList SplitAndCreateArrayList(string txt, int nrOfBlocksPrList)
        {
            //Strings longer than this can not be stored on the tag
            if (txt.Length > 252)
                txt = txt.Substring(0, 252);

            if (txt.Length <= 4 * nrOfBlocksPrList)
            {
                ArrayList list = new ArrayList();
                list.Insert(0,CreateArrayListFromString(txt));
                return list;
            }
            else
            {
                ArrayList list = SplitAndCreateArrayList(txt.Substring(4 * nrOfBlocksPrList), nrOfBlocksPrList);
                list.Insert(0,CreateArrayListFromString(txt.Substring(0, 4*nrOfBlocksPrList)));
                return list;
            }
        }

        public static ArrayList CreateArrayListFromString(string txt)
        {
            ArrayList list = new ArrayList();
            byte[] bytes = new byte[4];

            //add to length so txt.Length%4 ==0
            int stopAt = txt.Length;
            if(stopAt%4 != 0)
                stopAt=txt.Length + (4 - (txt.Length % 4));

            for (int i = 0; i < stopAt; i++)
            {
                if (i % 4 == 0)
                {
                    bytes = new byte[4];
                }
                //fill end with 0x00
                if (i >= txt.Length)
                    bytes[i % 4] = (byte)0x00;
                else
                    bytes[i % 4] = (byte)txt[i];

                if (i % 4 == 3)
                {
                    list.Add(new Block(bytes));
                }
            }
            return list;
        }
    }
}
