using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Cathexis.IDBlue;

namespace RFID_test
{
    public partial class Form1 : Form
    {
        private Cathexis.IDBlue.IDBlueNET _driver;

        public Form1()
        {
            InitializeComponent();
        }

        private void connectButton_Click(object sender, EventArgs e)
        {
            Append("Connecting...");
            Connect();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            Append("Loading drivers...");
            _driver = new Cathexis.IDBlue.IDBlueNET(
               Cathexis.IDBlue.IDBlueConnectionMethod.Franson);

            // _driver = new Cathexis.IDBlue.IDBlueNET(
            //     Cathexis.IDBlue.IDBlueConnectionMethod.Baracoda);

            // Device event handlers
            _driver.OnSearchComplete += new Cathexis.IDBlue.SearchComplete(_driver_OnSearchComplete);
            _driver.OnConnectFailed += new Cathexis.IDBlue.DeviceEvent(_driver_OnConnectFailed);
            _driver.OnDeviceActivated += new Cathexis.IDBlue.DeviceActivated(_driver_OnDeviceActivated);
            _driver.OnDeviceDeactivated += new Cathexis.IDBlue.DeviceDeactivated(_driver_OnDeviceDeactivated);
            
            
            // Tag event handlers
            _driver.OnAsyncButtonPress += new Cathexis.IDBlue.ButtonPressed(_driver_OnAsyncButtonPress);
            _driver.OnAsyncTagID += new Cathexis.IDBlue.AsyncTagID(_driver_OnAsyncTagID);
            //_driver.OnAsyncRead += new Cathexis.IDBlue.AsyncRead(_driver_OnAsyncRead);

            Append("driver successfully loaded.");
        }

        #region "IDBlue Event Handlers"

        void _driver_OnSearchComplete()
        {
            //this.UpdateDeviceList();
            MessageBox.Show("SearchComplete");
            Append("SearchComplete");
        }

        void _driver_OnConnectFailed()
        {
            //this.AddMessage("Connect attempt failed");
            //this.btnConnect.Text = "Connect";
            Append("Connect Failed");

        }

        void _driver_OnDeviceActivated()
        {
            //this.AddMessage("Connected established");

            this._driver.Timestamp = DateTime.Now;
            //this.AddMessage("Timestamp updated to " + DateTime.Now.ToShortTimeString());
            Append("Device activated");
        }

        void _driver_OnDeviceDeactivated()
        {
            //this.AddMessage("Connection lost");
            //this.btnConnect.Text = "Connect";
            //Append("Device deactivated");
        }


        /// <summary>
        /// TODO: describe usage of callback
        /// </summary>
        void _driver_OnAsyncButtonPress()
        {
            Append("ASYNC Button press received");

            Cathexis.IDBlue.SelectMediaObject tagObj;
            //ArrayList data;
            Cathexis.IDBlue.ErrorCode err;

            _driver.BeginCommands();

            // Attempt to scan an RFID tag
            err = _driver.ScanMediaId(out tagObj);
            if (err != ErrorCode.Success)
            {
                _driver.EndCommands(true);
                return;
            }

            //Read or write
            if (read.Checked)
            {

                // Read block data from this tag - note the use of passing
                // in the Media ID (tag ID) obtained from the ScanMediaId
                // method
                /*
                err = _driver.ReadBlocks(tagObj.MediaID, 0, 20, out data);

                if (err != ErrorCode.Success)
                {
                    _driver.EndCommands(true);
                    return;
                }
                */


                // Read block data from this tag - note the use of passing
                // in the Media ID (tag ID) obtained from the ScanMediaId
                // method
                
                
                ArrayList masterData = new ArrayList();
                ArrayList data = new ArrayList();
                //Read all 128 bytes (32 blocks)
                int i = 0;
                while(i<32)
                {
                    data = new ArrayList();
                    err = _driver.ReadBlocks(tagObj.MediaID, i, 16, out data);

                    if (err != ErrorCode.Success)
                    {
                        _driver.EndCommands(true);
                        Append("Error reading tag! i="+i);
                        return;
                    }
                    masterData.Add(data);
                    i+=16;
                    Append("Read 16 blocks of data");
                }

                // Successful end of RFID operations
                _driver.EndCommands(false);

                // Process the data in the data arraylist
                // Note that masterData is an ArrayList containing Arraylists
                String readData ="";
                foreach(ArrayList dataList in masterData)
                {
                    readData+=Utils.GetStringFromArrayList(dataList);
                }
                Append("Read data: \"" + readData+"\"");
                
            }
            else if (write.Checked && writeBox.Text.Length>0)
            {
                //Write som test data
                /*
                ArrayList writeData = Utils.CreateArrayListFromString(writeBox.Text);
                
                err = _driver.WriteBlocks(tagObj.MediaID, 0, writeData);

                if (err != ErrorCode.Success)
                {
                    _driver.EndCommands(true);
                    return;
                }
                */
                int i = 0;
                foreach (ArrayList writeData in Utils.SplitAndCreateArrayList(writeBox.Text, 12))
                {
                    err= _driver.WriteBlocks(tagObj.MediaID, i, writeData);
                    i += writeData.Count;
                    
                    if (err != ErrorCode.Success)
                    {
                        _driver.EndCommands(true);
                        Append("Error writing to tag!");
                        return;
                    }
                    Append("Wrote " + writeData.Count + " blocks");
                } 

                // Successful end of RFID operations
                _driver.EndCommands(false);

                // Process the data in the data arraylist
                Append("Wrote some data");
            }
            else if (clear.Checked)
            {
                int i = 0;
                foreach (ArrayList writeData in Utils.CreateClearTagList())
                {
                    err = _driver.WriteBlocks(tagObj.MediaID, i, writeData);
                    i += writeData.Count;

                    if (err != ErrorCode.Success)
                    {
                        _driver.EndCommands(true);
                        Append("Error writing to tag!");
                        return;
                    }
                    Append("Wrote " + writeData.Count + " blocks");
                }

                // Successful end of RFID operations
                _driver.EndCommands(false);

                // Process the data in the data arraylist
                Append("Cleared tag");
            }
            
        }

        /// <summary>
        /// TODO: describe usage of callback
        /// </summary>
        void _driver_OnAsyncTagID()
        {
            string tagId = _driver.AsyncData.TagId;
            Append("Received async tag ID: " + tagId);
        }

        /// <summary>
        /// TODO: describe usage of callback
        /// </summary>
        /*
        void _driver_OnAsyncRead()
        {
            string tagId = _driver.AsyncData.TagId;
            string data = _driver.AsyncData.data.ToString();

            this.AddMessage("Received async read block tag: " + tagId
                + ", data = " + data);
        }
        */

        #endregion

        private void Append(string txt)
        {
            console.Text += txt + "\r\n";
            //console.ScrollToCaret();
            //console.AppendText(txt);
        }

        private void Connect()
        {
            if (_driver.IsConnected)
            {
                Append("Disconnecting from device " + _driver.Device);
                _driver.Disconnect();

                connectButton.Text = "Connect";
            }
            else
            {
                // Set new device address
                //_driver.Device = devAddress;
                _driver.Device = "00:A0:96:0B:D5:71";

                Append("Connecting to device: " + _driver.Device);

                // Initiate a connection
                Cathexis.IDBlue.ErrorCode err = _driver.BeginConnection();
                if (err != Cathexis.IDBlue.ErrorCode.Success)
                {
                    MessageBox.Show("Error: could not initiate connection to device");
                    return;
                }

                connectButton.Text = "Disconnect";
            }
        }

        private void goButton_Click(object sender, EventArgs e)
        {
            _driver.ContinuousScanEnabled = false;
            _driver.SetConnectedMode(Cathexis.IDBlue.ConnectedOpMode.REACTIVE);
            Append("Connected operating mode set to:");
            Append("Button pressed / reaktive");
        }

        private void scanButton_Click_1(object sender, EventArgs e)
        {
            _driver.ContinuousScanEnabled = false;
            _driver.SetConnectedMode(Cathexis.IDBlue.ConnectedOpMode.TAGID);
            Append("Connected operating mode set to:");
            Append("Select Tag ID");
            Append("Ready to scan...");
        }

        private void write_CheckedChanged(object sender, EventArgs e)
        {
            if(write.Checked)
                writeBox.Enabled = true;
            else
                writeBox.Enabled = false;
        }

        private void Form1_Closing_1(object sender, CancelEventArgs e)
        {
            try
            {
                Append("Disconnecting from device " + _driver.Device);
                _driver.Disconnect();
                _driver.Dispose();
            }
            catch (Exception)
            {
            }
        }
    }
}