using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using RegisterLocation.DiscoveryService;
using RegisterLocation.PositioningService;
using RegisterLocation.LocationService;
using System.Collections;
using System.Net;

namespace RegisterLocation
{
    public partial class Form1 : Form
    {
        private PocketDiscoveryImplService discoveryService;
        private PositioningImplService positioningService;
        private LocationImplService locationService;

        private FileAccess fileAccess;

        public Form1()
        {
            InitializeComponent();
            fileAccess = new FileAccess();
            discoveryService = new PocketDiscoveryImplService();
            string[] positioningUrls = searchForServices("type:no.ubicollab.osgi.service.positioning protocol:ws");
            string[] locationUrls = searchForServices("type:no.ubicollab.osgi.service.location protocol:ws");
            if (positioningUrls.Length < 1 || locationUrls.Length < 1)
                MessageBox.Show("Could not find position service and/or location service");
            else
            {
                //use the first avilable position and location service
                positioningService = new PositioningImplService();
                locationService = new LocationImplService();

                //MessageBox.Show("old position ip " + positioningUrls[0]);
                //MessageBox.Show("old location ip " + locationUrls[0]);

                //positioningService.Url = swapWithLocalhostIfPossible(positioningUrls[0]);
                //locationService.Url = swapWithLocalhostIfPossible(locationUrls[0]);

                //MessageBox.Show("new position ip " + positioningService.Url);
                //MessageBox.Show("new location ip " + locationService.Url);

                positioningService.Url = positioningUrls[0];
                locationService.Url = locationUrls[0];
            }
        }
        /*
        private string swapWithLocalhostIfPossible(string serviceUrl)
        {
            //if the services are found on localhost, use 127.0.0.1 instead of the external ipaddress to allow the application to run without network connection
            //get local ip

            string localhost="";
            MessageBox.Show("Dns hostname: "+Dns.GetHostName());
            IPHostEntry host = Dns.GetHostEntry(Dns.GetHostName());
            foreach (IPAddress ip in host.AddressList)
            {
                localhost = ip.ToString();
                MessageBox.Show("localhost is " + localhost);
            }
            //if(host.AddressList.Length>0){
             //   localhost = host.AddressList[0].ToString();
            //}

            //MessageBox.Show("Localhost is " + localhost);

            Uri serviceUri = new Uri(serviceUrl);
            //get ipaddress part. 
            string serviceIP = serviceUri.DnsSafeHost;
            MessageBox.Show("DNS safe host is " + serviceIP);
            
            //make sure we get ip and not www.something.com
            //
            //IPHostEntry host2 = Dns.GetHostEntry(serviceIP);
            //if (host2.AddressList.Length > 0)
            //{
            //    serviceIP = host2.AddressList[0].ToString();
            //}
            
            MessageBox.Show("Service IP is " + serviceIP);

            if (localhost!="" && localhost.Equals(serviceIP))
            {
                return serviceUri.OriginalString.Replace(serviceUri.Host, "127.0.0.1");
            }
            //Else return original value
            else
                return serviceUrl;

        }
        */

        private String[] searchForServices(string searchParameter)
        {
            try
            {
                //searchResult should have the format: url protocol url protocol.....
                string searchResult = discoveryService.getService(searchParameter);
                if ("No matching services".Equals(searchResult) || "Error: Unknown service protocol".Equals(searchResult) || searchResult == null || searchResult.Length <= 0)
                {
                    return new string[] { };
                }
                else
                {
                    //Copy urls only
                    string[] urlAndProtocol = searchResult.Split(new Char[] { ' ' });
                    string[] urls = new string[urlAndProtocol.Length / 2];
                    for (int i = 0; i < urlAndProtocol.Length; i += 2)
                    {
                        urls[i / 2] = urlAndProtocol[i];
                    }
                    return urls;
                }
            }
            catch (Exception e)
            {
                MessageBox.Show("Could not search for services");
                MessageBox.Show(e.Message);
                MessageBox.Show(e.StackTrace);
                return new string[] { };
            }

        }

        private void button2_Click(object sender, EventArgs e)
        {
            try
            {
                if (positioningService != null)
                {
                    double[] pos = positioningService.getPosition(textBox2.Text);
                    if (pos != null)
                        listBox1.Items.Add(new PositionData(pos));
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Could not get position from positioning service: " + ex.Message);
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            PositionData posData = (PositionData)listBox1.SelectedItem;
            if (posData != null)
            {
                listBox1.Items.Remove(listBox1.SelectedItem);
            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            try
            {
                if (locationService != null && textBox1.Text != null && textBox1.Text.Length > 0 && listBox1.Items.Count > 0)
                {
                    ListBox.ObjectCollection oc = listBox1.Items;
                    double[] longitudes = new double[oc.Count];
                    double[] latitudes = new double[oc.Count];
                    int altitude = 0;

                    int i = 0;
                    foreach (PositionData posData in oc)
                    {
                        longitudes[i] = posData.Position[0];
                        latitudes[i] = posData.Position[1];
                        //I know :) altitude is overwritten in every iteration. Doesn't matter
                        altitude = (int)posData.Position[2];
                        i++;
                    }
                    locationService.addLocation(textBox1.Text, longitudes, latitudes, altitude);
                    MessageBox.Show("New location registered");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Could not register location in location service: " + ex.Message);
            }
        }

        private void menuItem6_Click(object sender, EventArgs e)
        {
            ListBox.ObjectCollection oc = listBox1.Items;
            String[] posDat = new String[oc.Count];
            int i=0;
            foreach (PositionData pos in oc)
            {
                posDat[i] = pos.ToString();
                i++;
            }
            fileAccess.StoreToFile(posDat);
            MessageBox.Show("Saved");
        }

        private void menuItem5_Click(object sender, EventArgs e)
        {
            ArrayList posDat = fileAccess.ReadFromFile();
            if(posDat!=null){
                foreach (String pos in posDat)
                {
                    listBox1.Items.Add(new PositionData(pos));
                }
                MessageBox.Show("Loaded");
            }
        }
    }

}