using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using Cathexis.IDBlue;
using System.Windows.Forms;
using System.Web.Services.Protocols;
using RemoteLogin.WebReference;


namespace RemoteLogin
{
    class RFID_scanner
    {
        private Form1 mother;
        private Cathexis.IDBlue.IDBlueNET _driver;

        public RFID_scanner(Form1 mother)
        {
            this.mother = mother;
            LoadDrivers();
        }

        public bool LoadDrivers()
        {
            _driver = new Cathexis.IDBlue.IDBlueNET(
               Cathexis.IDBlue.IDBlueConnectionMethod.Franson);

            // _driver = new Cathexis.IDBlue.IDBlueNET(
            //     Cathexis.IDBlue.IDBlueConnectionMethod.Baracoda);

            // Device event handlers
            _driver.OnConnectFailed += new Cathexis.IDBlue.DeviceEvent(_driver_OnConnectFailed);
            _driver.OnDeviceActivated += new Cathexis.IDBlue.DeviceActivated(_driver_OnDeviceActivated);
            _driver.OnDeviceDeactivated += new Cathexis.IDBlue.DeviceDeactivated(_driver_OnDeviceDeactivated);


            // Tag event handlers
            _driver.OnAsyncButtonPress += new Cathexis.IDBlue.ButtonPressed(_driver_OnAsyncButtonPress);
            _driver.OnAsyncTagID += new Cathexis.IDBlue.AsyncTagID(_driver_OnAsyncTagID);

            return true;
        }

        //Connects or disconnects the reader
        //returns true if the new status of the connection is "connected" false if the device was disconnected
        public bool Connect()
        {
            if (_driver.IsConnected)
            {
                _driver.Disconnect();

                return false;
            }
            else
            {
                // Set new device address
                //_driver.Device = devAddress;
                _driver.Device = "00:A0:96:0B:D5:71";

                //Append("Connecting to device: " + _driver.Device);

                // Initiate a connection
                Cathexis.IDBlue.ErrorCode err = _driver.BeginConnection();
                if (err != Cathexis.IDBlue.ErrorCode.Success)
                {
                    MessageBox.Show("Error: could not initiate connection to device");
                    return false;
                }

                return true;
            }
        }

        public void DisconnectAndClose()
        {
            try
            {
                _driver.Disconnect();
                _driver.Dispose();
            }
            catch (Exception)
            {
            }
        }

        public void SetInDataMode()
        {
            _driver.ContinuousScanEnabled = false;
            _driver.SetConnectedMode(Cathexis.IDBlue.ConnectedOpMode.REACTIVE);
        }

        public void SetInIdMode()
        {
            _driver.ContinuousScanEnabled = false;
            _driver.SetConnectedMode(Cathexis.IDBlue.ConnectedOpMode.TAGID);
        }


        #region "IDBlue Event Handlers"

        void _driver_OnConnectFailed()
        {
        }

        void _driver_OnDeviceActivated()
        {
            this._driver.Timestamp = DateTime.Now;
            SetInDataMode();
        }

        void _driver_OnDeviceDeactivated()
        {
        }


        void _driver_OnAsyncButtonPress()
        {

            Cathexis.IDBlue.SelectMediaObject tagObj;
            Cathexis.IDBlue.ErrorCode err;

            _driver.BeginCommands();

            // Attempt to scan an RFID tag
            err = _driver.ScanMediaId(out tagObj);
            if (err != ErrorCode.Success)
            {
                _driver.EndCommands(true);
                return;
            }

            // Read block data from this tag - note the use of passing
            // in the Media ID (tag ID) obtained from the ScanMediaId
            // method


            ArrayList masterData = new ArrayList();
            ArrayList data = new ArrayList();
            //Read all 128 bytes (32 blocks)
            int i = 0;
            while (i < 32)
            {
                data = new ArrayList();
                err = _driver.ReadBlocks(tagObj.MediaID, i, 16, out data);

                if (err != ErrorCode.Success)
                {
                    _driver.EndCommands(true);
                    //Append("Error reading tag! i=" + i);
                    return;
                }
                masterData.Add(data);
                i += 16;
                //Append("Read 16 blocks of data");
            }

            // Successful end of RFID operations
            _driver.EndCommands(false);

            // Process the data in the data arraylist
            // Note that masterData is an ArrayList containing Arraylists
            String url = "";
            foreach (ArrayList dataList in masterData)
            {
                url += Utils.GetStringFromArrayList(dataList);
            }

            try
            {
                LoginImplService loginService = new LoginImplService();
                loginService.Url = url;
                String result = loginService.login(Form1.Username, Form1.Password);
                MessageBox.Show(result);
            }
            catch (SoapException)
            {
               MessageBox.Show("Could not log in");
            }
            catch (Exception)
            {
                MessageBox.Show("Unknown Exception");
            }
        }


        /// <summary>
        /// TODO: describe usage of callback
        /// </summary>
        void _driver_OnAsyncTagID()
        {
            string tagId = _driver.AsyncData.TagId;
            //Append("Received async tag ID: " + tagId);
        }

        #endregion
    }
}
