/**
 * Title:       Binary Light PresentationPage Handler
 * Description: WebRequest Handler for Binary Light<p>
 * Copyright (c) 2001, Siemens AG<p>
 *
 * @author Markus Alexander Wischy
 * @version 1.0
 */

package com.siemens.upnp.testdevice;

import com.siemens.upnp.*;
import com.siemens.upnp.devicehost.*;
import com.siemens.upnp.testdevice.service.*;

import java.io.*;
import java.util.*;

public class BinaryLightPresentationPageHandler
        implements UPnPPresentationPageHandler {

    protected BinaryLightDevice device;
    protected SwitchPower powerService;

    protected final String ACTION_ON = "On";
    protected final String ACTION_OFF = "Off";

    public BinaryLightPresentationPageHandler(BinaryLightDevice device, SwitchPower powerService) {
        this.device = device;
        this.powerService = powerService;
    }

    public void processGetRequest(String path, Hashtable headers, BufferedOutputStream os) {

        // get print stream for string output
        PrintStream out = new PrintStream(os, false);

        // parse the URL path
        StringTokenizer tokenizer = new StringTokenizer(path, "/");

        // get the udn (first token of URL path)
        if (tokenizer.hasMoreTokens())
            tokenizer.nextToken();

        // get the action (second token of the URL path
        if (tokenizer.hasMoreTokens()) {
            String action = tokenizer.nextToken();

            if (action.equals(ACTION_ON)) {
                powerService.setTarget(true);
            }
            else if (action.equals(ACTION_OFF)) {
                powerService.setTarget(false);
            }
        }

        try {
            this.emitContent(out);
        }
        catch (IOException e) {
            out = null;
        }
    }

    protected void emitContent(PrintStream out) throws IOException {

        String nextAction;
        String action;

        if (powerService.getStatus() == true) {
            nextAction = this.ACTION_OFF;
            action = this.ACTION_ON;
        }
        else {
            nextAction = this.ACTION_ON;
            action = this.ACTION_OFF;
        }

        out.println("<HTML>");
        out.println("  <title>" + device.getFriendlyName() + "</title>");
        out.println("<BODY>");
        out.println("  <center>");
        out.println("  <h1><font face=\"Arial\" color=\"#808080\">"
                        + device.getFriendlyName() + "</font></h1>");
        out.println("<p><a href=/" + device.getPlainUDN() + "/" + nextAction + ">"
                    + "<img border=\"0\" src=\"/lamp" + action + ".gif\""
                    + "width=\"139\" height=\"196\"></a></p>");
        out.println("<p><a href=/" + device.getPlainUDN() + "/presentation.html>"
                    + "Refresh status</a></p>");
        out.println("  </center>");
        out.println("</BODY>");
        out.println("</HTML>");
        out.flush();
    }

    public void processPostRequest(String path, Hashtable headers,
        BufferedInputStream in, BufferedOutputStream os) {
    }

}