/**
 * Title:        Image button<p>
 * Description:  Button with to images, changes images when clicked
 * Copyright (c) 2000, Siemens AG<p>
 * @author Markus Alexander Wischy
 * @version 1.0
 */

package com.siemens.upnp.testdevice.ui;

import java.awt.Canvas;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Vector;

public class ImageButton extends Canvas implements MouseListener {

    Image imageOn;
    Image imageOff;
    boolean status = false;

    int imageWidth;
    int imageHeight;

    int imageX;
    int imageY;

    Vector listeners = new Vector(5);

    public ImageButton(Image imageOn, Image imageOff, boolean status) {
        this.imageOn = imageOn;
        this.imageOff = imageOff;
        this.status = status;
        imageWidth = imageOff.getWidth(this);
        imageHeight = imageOff.getHeight(this);
        this.setSize(imageWidth, imageHeight);
        this.addMouseListener(this);
    }

    public void setStatus(boolean newStatus) {
        if (status != newStatus) {
            this.status = newStatus;
            this.fireEvents();
            this.repaint();
        }
    }

    public boolean getStatus() {
        return status;
    }

    public void setBounds(int x, int y, int w, int h) {
        super.setBounds(x, y, w, h);
        this.calculateImagePos(w, h);
    }

/*    public void setBounds(Rectangle rect) {
        super.setBounds(rect);
        this.calculateImagePos(rect.getWidth(), rect.getHeight());
    }

    public void setSize(int w, int h) {
        super.setSize(w, h);
        this.calculateImagePos(w, h);
    }

    public void setSize(Dimension dim) {
        super.setSize(dim);
        this.calculateImagePos(dim.getWidth(), dim.getHeight());
    }
*/
    protected void calculateImagePos(int w, int h) {
        imageX = (w - imageWidth) / 2;
        imageY = (h - imageHeight) / 2;
        imageX = (imageX < 0) ? 0 : imageX;
        imageY = (imageY < 0) ? 0 : imageY;
    }

    public void paint(Graphics g) {
        if (status)
            g.drawImage(imageOn, imageX, imageY, this);
        else
            g.drawImage(imageOff, imageX, imageY, this);
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
        status = (status == true) ? false : true;
        this.fireEvents();
        this.repaint();
    }

    public void addChangeListener(ChangeListener listener) {
        listeners.addElement(listener);
    }

    protected void fireEvents() {
        for (int i=0; i < listeners.size(); i++) {
            ChangeListener listener =
                (ChangeListener)listeners.elementAt(i);
            listener.stateChanged(this, status);
        }
    }
}