import java.io.File;

import org.cybergarage.upnp.Action;
import org.cybergarage.upnp.Argument;
import org.cybergarage.upnp.ArgumentList;
import org.cybergarage.upnp.Device;
import org.cybergarage.upnp.Service;
import org.cybergarage.upnp.ServiceList;
import org.cybergarage.upnp.StateVariable;
import org.cybergarage.upnp.UPnPStatus;
import org.cybergarage.upnp.control.ActionListener;
import org.cybergarage.upnp.control.QueryListener;
import org.cybergarage.upnp.device.InvalidDescriptionException;

public class UbiLightDevice extends Device implements ActionListener{
	
	private final static String DESCRIPTION_FILE_NAME = "description/description.xml";
	
	private StateVariable statusVariable;
	private boolean onFlag=false;
	private UbiLight parent;
	
	public UbiLightDevice(UbiLight parent) throws InvalidDescriptionException
	{
		super(new File(DESCRIPTION_FILE_NAME));
		//setHTTPPort(8080);
		setHTTPPort(80);

		Action setTarget = getAction("SetTarget");
		setTarget.setActionListener(this);
		
		Action getStatus = getAction("GetStatus");
		getStatus.setActionListener(this);
		
		// serviceList = getServiceList();
		//Service service = serviceList.getService(0);
		//service.setQueryListener(this);
		statusVariable = getStateVariable("Status");
		
		this.parent=parent;
	}
	
	public void setPowerState(String newState){
		if("0".equals(newState) || "fale".equals(newState)){
			off();
		}
		else if("1".equals(newState) || "true".equals(newState)){
			on();
		}
		else{
			off();
		}
	}
	
	
	public String getPowerState(){
		if(onFlag)
			return "1";
		else
			return "0";
	}
	
	public void on(){
		onFlag=true;
		statusVariable.setValue("1");
		System.out.println("On");
		parent.turnOn();
	}
	public void off(){
		onFlag=false;
		statusVariable.setValue("0");
		System.out.println("Off");
		parent.turnOff();
	}

	public boolean actionControlReceived(Action action) {
		ArgumentList argList = action.getArgumentList();
		String actionName = action.getName();
		if (actionName.equals("SetTarget")) {
			Argument newStatus = argList.getArgument("newTargetValue");
			String newValue = newStatus.getValue();
			if (newValue == null || newValue.length() <= 0)
				return false;
			
			setPowerState(newValue);
			return true;
		}
		else if (actionName.equals("GetStatus")) {
			Argument resultStatus = argList.getArgument("ResultStatus");
			resultStatus.setValue(getPowerState());
			return true;
		}
		else
			return false;
	}

	/*
	public boolean queryControlReceived(StateVariable stateVar){
		stateVar.setValue(getPowerState());
		return true;
	}
	*/

	/**
	 * @param args
	 */
	
	/*
	public static void main(String[] args) {
		System.out.println("Starting...");
		try{
			
			UbiLightDevice uld=new UbiLightDevice();
			uld.start();
		}
		catch(InvalidDescriptionException ide){
			System.out.println("Invalid description file");
		}

	}
	*/

}
